/**************************************************************************
*   Copyright (C) 2009 by Ian Reinhart Geiser                             *
*   geiseri@kde.org                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#ifndef DESKTOPWIDGET_H
#define DESKTOPWIDGET_H

#include <QWidget>
#include <QHash>
#include <QPixmap>
#include <QPoint>

#include <KDE/KIcon>

class TopBar;
class BottomBar;
class QGridLayout;
class KDirWatch;

class DesktopIconWidget : public QWidget
{
    Q_OBJECT
public:
    DesktopIconWidget( const QString &path, QWidget *parent );
    void updateIcon();

protected:
    void mousePressEvent(QMouseEvent *);
    void mouseMoveEvent(QMouseEvent *);
    void mouseReleaseEvent(QMouseEvent *);
    void mouseDoubleClickEvent(QMouseEvent *);
    void paintEvent(QPaintEvent *);
private:
    QPixmap m_image;
    QString m_path;
    QString m_name;
    QPoint m_mousepos;
};

class DesktopWidget : public QWidget
{
    Q_OBJECT
public:
    DesktopWidget( QWidget *parent = 0 );
    ~DesktopWidget();
public slots:
    void slotUpdateStyleSheet( );

private slots:
    void slotDesktopResized ( int screen );
    void slotFileCreated( const QString &path );
    void slotFileDeleted( const QString &path );
    void slotFileChanged( const QString &path );

private:
    void populateDesktop( const QString &path );

    TopBar *m_topBar;
    BottomBar *m_bottomBar;
    const int m_size;
    QHash<QString,DesktopIconWidget*> m_icons;
    QPoint m_lastPos;
    KDirWatch *m_desktopWatcher;
    KDirWatch *m_styleWatcher;
};

#endif // DESKTOPWIDGET_H
