/**************************************************************************
*   Copyright (C) 2009 by Ian Reinhart Geiser                             *
*   geiseri@kde.org                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "mediamenu.h"

#include <KDE/KIcon>
#include <KDE/KAction>
#include <KDE/KToolInvocation>
#include <KDE/KRun>
#include <KDE/KLocale>

#include <solid/devicenotifier.h>
#include <solid/deviceinterface.h>
#include <solid/device.h>
#include <solid/storagedrive.h>
#include <solid/storageaccess.h>
#include <solid/storagevolume.h>

#include <QTimer>

#include <kdebug.h>

MediaMenu::MediaMenu( QWidget *parent ) : KMenu(i18n("Media"),parent)
{
    setObjectName("MediaMenu");
    connect(Solid::DeviceNotifier::instance(), SIGNAL(deviceAdded(const QString&)),
            this, SLOT(slotRebuildMenus()));
    connect(Solid::DeviceNotifier::instance(), SIGNAL(deviceRemoved(const QString&)),
            this, SLOT(slotRebuildMenus()));

    connect(this, SIGNAL(triggered(QAction*)), this, SLOT(slotActionTriggered(QAction*)));
    QTimer::singleShot(0, this, SLOT(slotRebuildMenus()));
}

void MediaMenu::slotActionTriggered ( QAction * action )
{
    Solid::Device device( action->data().toString() );
    if( device.isValid() )
    {
        Solid::StorageAccess *vol = device.as<Solid::StorageAccess>();
        if( vol )
        {
            if( vol->isAccessible() )
                KRun::runUrl(vol->filePath(), "inode/directory", this);
            else if( vol->setup() )
            {
                connect( vol, SIGNAL(setupDone(Solid::ErrorType,QVariant,QString)),
                         this, SLOT(executeFileManager(Solid::ErrorType,QVariant,QString)));
            }

        }
    }
}

void MediaMenu::executeFileManager(Solid::ErrorType error,const QVariant &message , const QString &udi)
{
    if( error == Solid::NoError )
    {
        Solid::Device device( udi );
        if( device.isValid() )
        {
            Solid::StorageAccess *vol = device.as<Solid::StorageAccess>();
            if( vol )
            {
                KRun::runUrl(vol->filePath(), "inode/directory", this);
            }
        }
    }
}

void MediaMenu::slotRebuildMenus( )
{
    bool hasItems = false;
    clear();
    foreach( Solid::Device device, Solid::Device::listFromType(Solid::DeviceInterface::StorageVolume, QString()))
    {        
        Solid::StorageDrive *drive = 0;
        Solid::Device parentDevice = device;
        while (parentDevice.isValid() && !drive) {
            drive = parentDevice.as<Solid::StorageDrive>();
            parentDevice = parentDevice.parent();
        }

        if (drive && (drive->isHotpluggable() || drive->isRemovable()))
        {
            QString udi = device.udi();
            QString diskname = device.as<Solid::StorageVolume>()->label();
            if( diskname.isEmpty())
            {
                diskname = device.product();
            }
            KAction *act = new KAction(KIcon( device.icon() ), diskname, this);
            act->setData( udi );
            addAction( act );
            hasItems = true;
        }
    }

    emit itemsAvailable(hasItems);
}

