/**************************************************************************
*   Copyright (C) 2009 by Ian Reinhart Geiser                             *
*   geiseri@kde.org                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "systemtray.h"

#include <QX11EmbedWidget>
#include <QDesktopWidget>
#include <QHBoxLayout>
#include <QX11Info>

#include <kwindowsystem.h>
#include <KDE/KApplication>

#include <X11/Xlib.h>
#define SYSTEM_TRAY_REQUEST_DOCK    0
#define SYSTEM_TRAY_BEGIN_MESSAGE   1
#define SYSTEM_TRAY_CANCEL_MESSAGE  2

Atom net_opcode_atom;
Atom net_selection_atom;
Atom net_manager_atom;
Atom net_message_data_atom;

SystemTray::SystemTray(QWidget *parent) : QFrame(parent)
{
    setObjectName("SystemTray");
    m_layout = new QHBoxLayout(this);
    m_layout->setDirection(QBoxLayout::RightToLeft);
    m_layout->setContentsMargins(0, 0, 0, 0);
    m_layout->setSpacing(1);

    setLayout(m_layout);

     // Freedesktop.org system tray support
    char name[20] = {0};
    qsnprintf(name, 20, "_NET_SYSTEM_TRAY_S%d", DefaultScreen(QX11Info::display()));
    net_selection_atom = XInternAtom(QX11Info::display(), name, FALSE);
    net_opcode_atom = XInternAtom(QX11Info::display(), "_NET_SYSTEM_TRAY_OPCODE", FALSE);
    net_manager_atom = XInternAtom(QX11Info::display(), "MANAGER", FALSE);
    net_message_data_atom = XInternAtom(QX11Info::display(), "_NET_SYSTEM_TRAY_MESSAGE_DATA", FALSE);

    XSetSelectionOwner(QX11Info::display(), net_selection_atom, winId(), CurrentTime);

    if (XGetSelectionOwner(QX11Info::display(), net_selection_atom) == winId())
    {
        XClientMessageEvent xev;

        xev.type = ClientMessage;
        xev.window = QApplication::desktop()->winId();
        xev.message_type = net_manager_atom;
        xev.format = 32;
        xev.data.l[0] = CurrentTime;
        xev.data.l[1] = net_selection_atom;
        xev.data.l[2] = winId();
        xev.data.l[3] = 0;        /* manager specific data */
        xev.data.l[4] = 0;        /* manager specific data */

        XSendEvent(QX11Info::display(), QApplication::desktop()->winId(), False, StructureNotifyMask, (XEvent *)&xev);
    }

}

void SystemTray::systemTrayWindowAdded( WId id )
{
    QX11EmbedContainer *item = new QX11EmbedContainer(this);
    item->embedClient(id);

    m_layout->addWidget(item);
    item->setContentsMargins(0, 0, 0, 0);
    item->setFixedSize( height(), height()  );
    XResizeWindow(QX11Info::display(), id, item->width(), item->height());
    //XMapRaised(QX11Info::display(), id);

    connect(item, SIGNAL(clientClosed()), item, SLOT(deleteLater()));
}

void SystemTray::systemTrayWindowRemoved( )
{

}

bool SystemTray::x11Event(XEvent *event)
{
    if (event->type == ClientMessage)
    {
        if (event->xclient.message_type == net_opcode_atom && event->xclient.data.l[1] == SYSTEM_TRAY_REQUEST_DOCK)
        {
            systemTrayWindowAdded(event->xclient.data.l[2]);
        }
        else if(event->xclient.message_type == net_opcode_atom && event->xclient.data.l[1] == SYSTEM_TRAY_BEGIN_MESSAGE)
        {

        }
        else if(event->xclient.message_type == net_opcode_atom && event->xclient.data.l[1] == SYSTEM_TRAY_CANCEL_MESSAGE)
        {

        }
        else if(event->xclient.data.l[1] == net_message_data_atom)
        {

        }
        return true;
    }
    return false;
}

