/**************************************************************************
*   Copyright (C) 2009 by Ian Reinhart Geiser                             *
*   geiseri@kde.org                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "taskmanagerwidget.h"
#include "taskitembutton.h"

#include <KDE/KApplication>
#include <KDE/KAboutData>
#include <KDE/KCmdLineArgs>
#include <KDE/KServiceGroup>
#include <KDE/KIcon>
#include <KDE/KWindowInfo>
#include <taskmanager/taskgroup.h>
#include <taskmanager/taskitem.h>

#include <QHBoxLayout>
#include <QLabel>
#include <QButtonGroup>
#include <QTimer>

#include <kdebug.h>

TaskManagerWidget::TaskManagerWidget( QWidget *parent ) : QFrame( parent )
{
    setObjectName("TaskBar");
    m_buttonGroup = new QButtonGroup(this);
    m_buttonGroup->setExclusive(true);

    m_manager = new TaskManager::GroupManager(this);

    TaskManager::TaskManager* mgr = TaskManager::TaskManager::self();
    connect( mgr, SIGNAL(startupAdded(StartupPtr)),
             m_manager, SLOT(addStartup(StartupPtr)));
    connect( mgr, SIGNAL(startupRemoved(StartupPtr)),
             m_manager, SLOT(removeStartup(StartupPtr)));
    connect( mgr, SIGNAL(taskAdded(TaskPtr)),
             m_manager, SLOT(addTask(TaskPtr)));
    connect( mgr, SIGNAL(taskRemoved(TaskPtr)),
             m_manager, SLOT(removeTask(TaskPtr)));

    connect( m_manager->rootGroup(), SIGNAL(itemAdded(AbstractGroupableItem*)),
             this, SLOT(onItemAdded(AbstractGroupableItem*)));
    connect( m_manager->rootGroup(), SIGNAL(itemRemoved(AbstractGroupableItem*)),
             this, SLOT(onItemRemoved(AbstractGroupableItem*)));

    m_taskLayout = new QHBoxLayout(this);
    m_taskLayout->setMargin(0);
    m_taskLayout->setSpacing(0);

    // If we don't wait, we get some ghost tasks which crash us
    QTimer::singleShot(1000, this, SLOT(addInitialTasks()));
}

void TaskManagerWidget::addInitialTasks()
{
    foreach(TaskPtr task, TaskManager::TaskManager::self()->tasks()){
        TaskManager::TaskItem *taskItem = new TaskManager::TaskItem(this, task);
        onItemAdded(taskItem);
    }
}

QSize TaskManagerWidget::minimumSizeHint () const
{
    QSize sz = QFrame::minimumSizeHint();

    int goodWidth = 0;
    foreach( TaskItemButton *cell, m_buttons )
    {
        goodWidth = qMax( goodWidth,  cell->minimumSizeHint().width() );
    }
    sz.setWidth( goodWidth * m_buttons.count() );

    return sz;
}

void TaskManagerWidget::onItemAdded (AbstractGroupableItem *item)
{
    m_buttons[item] = new TaskItemButton(item, m_manager, this);
    m_buttonGroup->addButton(m_buttons[item]);
    m_taskLayout->addWidget(m_buttons[item],1);
}

void TaskManagerWidget::onItemRemoved (AbstractGroupableItem *item)
{
    delete m_buttons[item];
    m_buttons.remove(item);
}

