/*
 * KDE Composite Manager, 
 * Copyright (C) 2003-2005 Fredrik Höglund <fredrik@kde.org>
 *
 * Based on xcompmgr,
 * Copyright (C) 2003 Keith Packard <keithp@keithp.com>
 *
 * Internal structure and naming style inspired by KWin,
 * Copyright (C) 1999, 2000 Matthias Ettrich <ettrich@kde.org>
 * Copyright (C) 2003 Lubos Lunak <l.lunak@kde.org>
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Fredrik Höglund not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  Fredrik Höglund makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * FREDRIK HÖGLUND DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL FREDRIK HÖGLUND BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */


#include <qrect.h>

#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/extensions/Xrender.h>
#include <X11/extensions/Xfixes.h>

#include "utils.h"

extern Display *dpy;

 
Window getParentWindow( Window wId )
{
	Window root, parent = None, *children = 0;
	uint nchildren;

	if ( XQueryTree( dpy, wId, &root, &parent, &children, &nchildren ) == Success
			&& children && nchildren > 0 )
		XFree( children );

	return parent;
}


void *readXProperty( Window window, Atom property, Atom type, int *items )
{
	long offset = 0, length = 2048l;
	Atom actual_type;
	int actual_format;
	unsigned long nitems, bytes_remaining;
	unsigned char *data = 0l;

	int result = XGetWindowProperty( dpy, window, property, offset, length, false, type,
						 &actual_type, &actual_format, &nitems, &bytes_remaining, &data );

	if ( result == Success && actual_type == type && actual_format == 32 && nitems > 0 )
	{
		if ( items )
			*items = nitems;

		return reinterpret_cast< void* >( data );
	}

	if ( data )
		XFree( data );

	if ( items )
		*items = 0;

	return NULL;
}


bool readCardinalProperty( Window window, Atom property, int *value )
{
	void *data = readXProperty( window, property, XA_CARDINAL, NULL );

	if ( data )
	{
		if ( value )
			*value = *reinterpret_cast<int *>( data );
		XFree( data );
		
		return true;
	}

	return false;
}


bool readWindowProperty( Window window, Atom property, Window *value )
{
	void *data = readXProperty( window, property, XA_WINDOW, NULL );

	if ( data )
	{
		if ( value )
			*value = *reinterpret_cast<int *>( data );
		XFree( data );

		return true;
	}

	return false;
}

bool readPixmapProperty( Window window, Atom property, Pixmap *value )
{
	void *data = readXProperty( window, property, XA_PIXMAP, NULL );

	if ( data )
	{
		if ( value )
			*value = *reinterpret_cast<int *>( data );
		XFree( data );

		return true;
	}

	return false;
}

Picture createPicture( int width, int height, int fmt, uint flags )
{
	XRenderPictFormat *format = XRenderFindStandardFormat( dpy, fmt );
	XRenderPictureAttributes pa;
	pa.repeat = bool( flags & Repeating );

	Pixmap pixmap = XCreatePixmap( dpy, DefaultRootWindow( dpy ), width, height, 8 );
	Picture picture = XRenderCreatePicture( dpy, pixmap, format, CPRepeat, &pa );
	XFreePixmap( dpy, pixmap );

	return picture;
}

