/***************************************************************************
 *   Copyright (C) 2008-2009 by Marcel Hasler <mahasler_at_gmail.com>      *
 *   Copyright (C) 2010 by Ozan Çağlayan <ozan_at_pardus.org.tr>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "osd.h"

#include <QWidget>
#include <QApplication>
#include <QGraphicsLinearLayout>

#include <KIconLoader>

#include <Plasma/Theme>


#define OSD_BASE_WIDTH 900
#define OSD_BASE_HEIGHT 400

#define ICON_BASE_SIZE 200
#define ICON_BASE_X ((OSD_BASE_WIDTH/2.0)-(ICON_BASE_SIZE/2.0))
#define ICON_BASE_Y 50

#define LABEL_BASE_WIDTH 470
#define LABEL_BASE_HEIGHT 200
#define LABEL_BASE_X 320
#define LABEL_BASE_Y 50

#define METER_BASE_WIDTH 700
#define METER_BASE_HEIGHT 60
#define METER_BASE_X 100
#define METER_BASE_Y 280

#define FONT_BASE_SIZE 65

OSD::OSD()
: Plasma::Dialog(0, Qt::Window | Qt::WindowStaysOnTopHint | Qt::FramelessWindowHint | Qt::X11BypassWindowManagerHint)
, m_backPanel(this)
, m_meter(new Plasma::Meter)
, m_timer(this)
, m_hPos(50)
, m_vPos(70)
, m_timeout(2)
, m_scaleFactor(0.3f)
, m_opacity(0.9f)
, m_isVisible(false)
, m_lockPrimaryScreen(false)
{
    // Load theme
    slotReloadTheme();
    connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), this, SLOT(slotReloadTheme()));

    // Set timer
    m_timer.setSingleShot(true);
    connect(&m_timer, SIGNAL(timeout()), this, SLOT(slotHide()));

    // Set meter
    m_meter->setMeterType(Plasma::Meter::BarMeterHorizontal);
    m_meter->setMaximum(100);
    m_meter->setMaximumHeight(ICON_BASE_Y);

    setAttribute(Qt::WA_TranslucentBackground);

    //setWindowOpacity(1.0);
    initLayout();
}

OSD::~OSD()
{
    // Destructor
}


void OSD::setSize(unsigned int percent)
{
    if (percent > 100)
        percent = 100;
    else if (percent < 10)
        percent = 10;

    m_scaleFactor = (float)percent / 100.0f;
    initLayout();
}


void OSD::setPosition(unsigned int xPercent, unsigned int yPercent)
{
    m_hPos = (xPercent > 100)?100:xPercent;
    m_vPos = (yPercent > 100)?100:yPercent;
}


void OSD::setOpacity(unsigned int percent)
{
    percent = (percent > 100)?100:percent;

    m_opacity = (float)percent / 100.0f;
    //setWindowOpacity(m_opacity);
}


void OSD::setPrimaryScreenLock(bool lock)
{
    m_lockPrimaryScreen = lock;
}


void OSD::setTimeout(unsigned int seconds)
{
    m_timeout = (seconds < 1)?1:seconds;
}


void OSD::updatePosition()
{
    QRect rect;
    if (m_lockPrimaryScreen)
        rect = QApplication::desktop()->screenGeometry(QApplication::desktop()->primaryScreen());
    else
        rect = QApplication::desktop()->screenGeometry(QCursor::pos());

    m_xPos = rect.x() + (unsigned int)((rect.width() - m_width) / 100.0f * m_hPos);
    m_yPos = rect.y() + (unsigned int)((rect.height() - m_height) / 100.0f * m_vPos);
    setGeometry(m_xPos, m_yPos, m_width, m_height);
}


void OSD::initLayout()
{
    m_width = (unsigned int)(OSD_BASE_WIDTH * m_scaleFactor);
    m_height = (unsigned int)(OSD_BASE_HEIGHT * m_scaleFactor);

    QGraphicsWidget *topWidget;

    topWidget = new QGraphicsWidget();

    QGraphicsLinearLayout *mainLayout;
    QGraphicsLinearLayout *topLayout;

    mainLayout = new QGraphicsLinearLayout(Qt::Vertical);
    topLayout = new QGraphicsLinearLayout(Qt::Horizontal);

    topLayout->addStretch();

    m_labelIcon = new Plasma::IconWidget();
    topLayout->addItem(m_labelIcon);

    m_labelText = new Plasma::Label();
    topLayout->addItem(m_labelText);

    topLayout->addStretch();

    mainLayout->addItem(topLayout);
    mainLayout->addItem(m_meter);

    setGraphicsWidget(topWidget);
    graphicsWidget()->setLayout(mainLayout);

    //m_backPanel.resizeFrame(QSize(m_width, m_height));
    //setMask(m_backPanel.mask());
}


void OSD::display(QString icon, QString text, unsigned int percent)
{
    updatePosition();

    m_labelIcon->setIcon(icon);
    //m_labelIcon->setIcon(KIconLoader::global()->loadIcon(icon, KIconLoader::NoGroup, 32));
    m_labelText->setText(text);

    m_meter->setValue((percent > 100)?100:percent);

    this->show();
    this->update();

    m_isVisible = true;
    m_timer.start(m_timeout*1000);
}


void OSD::slotHide()
{
    hide();
    m_isVisible = false;
}


void OSD::slotReloadTheme()
{
    // Load back panel
    m_backPanel.setImagePath("dialogs/background");
    /* FIXME: kdDebug()?
    if (!m_backPanel.isValid())
        cerr << "Back panel SVG is not valid!" << endl;
    */

    // Get the standard plasma font
    m_labelFont = Plasma::Theme::defaultTheme()->font(Plasma::Theme::DefaultFont);
    m_labelFont.setBold(true);

    m_labelColor = Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor);

}

#include "osd.moc"
