/*
    This file is part of KDE.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "installer.h"

#include "entry.h"

#include "kupdateiface_stub.h"

#include <kmessagebox.h>
#include <klocale.h>
#include <kprocess.h>
#include <kdebug.h>
#include <kapplication.h>
#include <dcopclient.h>

#include <qwidget.h>
#include <qtimer.h>

Installer::Installer( Engine *engine, QWidget *widget )
  : mEngine( engine ), mWidget( widget )
{
}

void Installer::install( KNS::Entry *entry )
{
  mAppname = entry->name();

  KURL url = entry->payload();

  kdDebug() << "Installer::install() " << url << endl;

  KMessageBox::information( mWidget, i18n("Install '%1'").arg( url.url() ) );

  doInstall( entry );
}

void Installer::finishInstallation()
{
  KMessageBox::information( mWidget, i18n("Installation complete.") );
  emit installationFinished();
  
  // do this on a timer to give us time to return true
  QTimer::singleShot(0, this, SLOT(slotRestart()));
}

void Installer::slotRestart()
{
  DCOPClient *c = kapp->dcopClient();
  if ( !c->isApplicationRegistered( mAppname.utf8() ) ) {
    kdDebug() << "Application " << mAppname << " isn't running." << endl;
  } else {
    KUpdateIface_stub kupdater( mAppname.utf8(), "KUpdate" );
    kupdater.restart();
    if ( !kupdater.ok() ) {
      KMessageBox::sorry( mWidget, i18n("Unable to call restart() via DCOP") );
    }
  }
}

#include "installer.moc"
