/*******************************************************************************
 * This file is a part of KDE project                                          *
 * klbSearchbar is a toolbar plugin for konqueror to use search functionality  *
 * of klb                                                                      *
 *                                                                             *
 * Copyright (C) 2005  Ramakrishna R <coolrama@gmail.com>                      *
 *                                                                             *
 * This program is free software; you can redistribute it and/or modify        *
 * it under the terms of the GNU General Public License as published by        *
 * the Free Software Foundation; either version 2 of the License, or           *
 * (at your option) any later version.                                         *
 *                                                                             *
 * This program is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                *
 * GNU General Public License for more details.                                *
 *                                                                             *
 * You should have received a copy of the GNU General Public License           *
 * along with this program; if not, write to the Free Software                 *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. *
 *                                                                             *
 ******************************************************************************/

#include <qtimer.h>
#include "klbsearchbar.h"
#include <kapplication.h>
#include <kgenericfactory.h>
#include <khtml_part.h>
#include <kdebug.h>
#include <kaction.h>
#include <kparts/partmanager.h>

typedef KGenericFactory<KlbSearchBarPlugin> KlbSearchBarPluginFactory;
K_EXPORT_COMPONENT_FACTORY(libklbsearchbarplugin,
                           KlbSearchBarPluginFactory("klbsearchbarplugin"))

KlbSearchBarPlugin::KlbSearchBarPlugin(QObject *parent, const char *name,
                                 const QStringList &) :
  KParts::Plugin(parent, name),
  _searchCombo(0)   
{
	_searchCombo = new KlbSearchBarc(0L, "klb search");    
	_searchCombo->setFixedWidth(210);    

	_searchComboAction = new KWidgetAction(_searchCombo, i18n("KLB Search Bar"), 0,
	                                                     0, 0, actionCollection(), "klb_search_bar");

	connect(_searchCombo, SIGNAL(activated(const QString &)),
	                       SLOT(search(const QString &)));

	KParts::MainWindow *mainWin = static_cast<KParts::MainWindow*>(parent);

	KParts::PartManager *partMan = static_cast<KParts::PartManager*>(mainWin->child(0, "KParts::PartManager"));
	if (partMan)
	{
		connect(partMan, SIGNAL(activePartChanged(KParts::Part*)),
		                 SLOT  (partChanged      (KParts::Part*)));
		partChanged(partMan->activePart());
	}
}

KlbSearchBarPlugin::~KlbSearchBarPlugin()
{
	delete _searchCombo;
	_searchCombo = 0L;
}

void KlbSearchBarPlugin::search(const QString &search)
{
	if( search.isEmpty() || !_part)
		return;
	{
			KParts::URLArgs args;
			args.setNewTab(true);
			QString searchstring = "klb:/view?searchstring=";
			searchstring = searchstring + search;
			emit _part->browserExtension()->createNewWindow(searchstring , args );
	}
	_searchCombo->setCurrentText("");
}

void KlbSearchBarPlugin::partChanged(KParts::Part *newPart)
{
	_part = ::qt_cast<KHTMLPart*>(newPart);

	// this is required or else it will be visible even in list view
	QTimer::singleShot(0, this, SLOT(updateVisibility()));
}


void KlbSearchBarPlugin::updateVisibility()
{
	if (_part.isNull() || !_searchComboAction->isPlugged())
	{
		_searchCombo->setPluginActive(false);
		_searchCombo->hide();
	}
	else
	{
		_searchCombo->setPluginActive(true);
		_searchCombo->show();
	}
}

KlbSearchBarc::KlbSearchBarc(QWidget *parent, const char *name) :
  KHistoryCombo(parent, name),
  _pluginActive(true)
{
}

void KlbSearchBarc::setPluginActive(bool pluginActive)
{
	_pluginActive = pluginActive;
}

void KlbSearchBarc::show()
{
	if(_pluginActive)
	{
		KHistoryCombo::show();
	}
}

#include "klbsearchbar.moc"
