/*
 *   Copyright © 2009 Dmitry Suzdalev (dimsuz@gmail.com)
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#ifndef CLIPBOARDTYPES_H
#define CLIPBOARDTYPES_H

#include "klipper_export.h"

#include <QVariant>

struct KLIPPER_EXPORT ClipboardEntry
{
    ClipboardEntry()
        : data(QVariant()), uuid(QString()), isText(true),
        isSelection(false)
    {}

    bool isValid() const { return data.isValid() && !uuid.isNull(); }

    // clipboard data itself - a text or an image
    QVariant data;
    // unique id of this entry
    QString uuid;
    // indicates if this entry is a text or an image
    bool isText;
    // indicates if this entry was copied to selection buffer (true)
    // or a clipboard buffer (false)
    bool isSelection;
};

typedef QList<ClipboardEntry> ClipboardEntryList;

inline bool operator==(const ClipboardEntry& entry1, const ClipboardEntry& entry2)
{
    // if uuids are not empty compare by uuids, otherwise - compare data
    if (entry1.uuid.isEmpty() || entry2.uuid.isEmpty())
        return entry1.data == entry2.data;
    else
        return entry1.uuid == entry2.uuid;
}


#endif
