#include "imagebrowser.h"

#include <KLocale>
#include <KDirModel>
#include <KDirLister>
#include <KFileItemDelegate>
#include <KIconLoader>
#include <KLineEdit>

#include <QListView>
#include <QVBoxLayout>
#include <QList>

#include <Nepomuk/Query/FileQuery>
#include <Nepomuk/Query/QueryParser>
#include <Nepomuk/Query/AndTerm>
#include <Nepomuk/Query/ResourceTypeTerm>
#include <Nepomuk/Query/ResourceTerm>
#include <Nepomuk/Query/ComparisonTerm>
#include <Nepomuk/TagWidget>
#include <Nepomuk/Tag>

#include <Soprano/Vocabulary/NAO>

#include "nfo.h"


ImageBrowser::ImageBrowser()
    : KMainWindow()
{
    QWidget* w = new QWidget(this);
    m_queryEdit = new KLineEdit(w);
    connect(m_queryEdit, SIGNAL(returnPressed()), this, SLOT(slotUpdateQuery()) );
    m_tagWidget = new Nepomuk::TagWidget(w);
    connect(m_tagWidget, SIGNAL(selectionChanged(QList<Nepomuk::Tag>)),
            this, SLOT(slotUpdateQuery()));
    QListView* view = new QListView( w );
    QVBoxLayout* layout = new QVBoxLayout(w);
    layout->addWidget(m_queryEdit);
    layout->addWidget(m_tagWidget);
    layout->addWidget(view);
    setCentralWidget(w);

    view->setIconSize( QSize(KIconLoader::SizeMedium, KIconLoader::SizeMedium) );

    KFileItemDelegate* delegate = new KFileItemDelegate(view);
    delegate->setShowInformation(KFileItemDelegate::InformationList() << KFileItemDelegate::FriendlyMimeType );
    view->setItemDelegate(delegate);

    m_model = new KDirModel( this );
    view->setModel(m_model);

    Nepomuk::Query::FileQuery query;
    query.setTerm( Nepomuk::Query::ResourceTypeTerm( Nepomuk::Vocabulary::NFO::RasterImage() ) );
//    query.setLimit(-1);
    m_model->dirLister()->openUrl(query.toSearchUrl());
}

ImageBrowser::~ImageBrowser()
{
}


void ImageBrowser::slotUpdateQuery()
{
    Nepomuk::Query::FileQuery query = Nepomuk::Query::QueryParser::parseQuery( m_queryEdit->text() );
    Nepomuk::Query::AndTerm andTerm;
    andTerm.addSubTerm(query.term());
    andTerm.addSubTerm(Nepomuk::Query::ResourceTypeTerm( Nepomuk::Vocabulary::NFO::RasterImage() ));
    Q_FOREACH(const Nepomuk::Tag& tag, m_tagWidget->selectedTags())
        andTerm.addSubTerm(Nepomuk::Query::ComparisonTerm(Soprano::Vocabulary::NAO::hasTag(), Nepomuk::Query::ResourceTerm(tag)));
    query.setTerm(andTerm);
    m_model->dirLister()->openUrl(query.toSearchUrl());
}

#include "imagebrowser.moc"
