/*
    This file is part of the Nepomuk KDE project.
    Copyright (C) 2010  Vishesh Handa <handa.vish@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef BACKUPWIZARDPAGES_H
#define BACKUPWIZARDPAGES_H

#include <QtGui/QWizardPage>
#include <QtGui/QRadioButton>
#include <QtGui/QProgressBar>

#include "backupmanagerinterface.h"

#include "ui_intropage.h"
#include "ui_backupoptions.h"
#include "ui_backuppage.h"
#include "ui_checkpointpage.h"

namespace Nepomuk {

    class BackupWizard;

    class IntroPage : public QWizardPage, public Ui::IntroPage {
    public:
        IntroPage(QWidget* parent = 0);

        virtual int nextId() const;
    private:
    };

    class BackupOptionsPage : public QWizardPage, public Ui::BackupOptionsPage {
    public:
        BackupOptionsPage(QWidget* parent = 0);

        virtual int nextId() const;
    private:

    };

    class BackupPage : public QWizardPage, public Ui::BackupPage {
        Q_OBJECT
    public:
        BackupPage(QWidget* parent = 0);

        virtual void initializePage();
        virtual int nextId() const;
    private:
        
        typedef org::kde::nepomuk::services::nepomukbackupsync::BackupManager BackupManager;
        BackupManager* m_backupManager;

    private slots:
        void slotBackupDone();
        void slotBackupNotRequired( QDateTime dt );
    };

    class CheckpointPage : public QWizardPage, public Ui::CheckpointPage {
        Q_OBJECT
    public:
        CheckpointPage(QWidget* parent = 0);
        
        virtual void initializePage();
        virtual int nextId() const;
    private:
        typedef org::kde::nepomuk::services::nepomukbackupsync::BackupManager BackupManager;
        BackupManager* m_backupManager;
        
    private slots:
        void revert();
    };


    class IdentifierWidget;
    class MergerWidget;
    
    class RestorePage : public QWizardPage {
        Q_OBJECT
    public:
        RestorePage( QWidget * parent );

        virtual void initializePage();
        virtual int nextId() const;
    private:
        IdentifierWidget* m_identifier;
        MergerWidget* m_merger;

        typedef org::kde::nepomuk::services::nepomukbackupsync::BackupManager BackupManager;
        BackupManager* m_backupManager;
    };
}

#endif // BACKUPWIZARDPAGES_H
