/*
    This file is part of the Nepomuk KDE project.
    Copyright (C) 2010  Vishesh Handa <handa.vish@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef IDENTIFICATIONTHREAD_H
#define IDENTIFICATIONTHREAD_H

#include <QtCore/QThread>
#include <QtCore/QQueue>
#include <QtCore/QMutex>
#include <QtCore/QWaitCondition>
#include <QtCore/QQueue>
#include <QtCore/QUrl>

#include "syncfileidentifier.h"

namespace Soprano {
    class Model;
}

namespace Nepomuk {

    class SyncFile;
    class ChangeLog;
    
    class Identifier : public QThread
    {
        Q_OBJECT

    public :
        Identifier( QObject* parent = 0 );
        virtual ~Identifier();

        void stop();
        void test();

    Q_SIGNALS:
        void identified( int id, const QString & oldUri, const QString & newUri );
        void notIdentified( int id, const QString & serializedStatements );
        
        void processed( const Nepomuk::ChangeLog & logFile );

    public Q_SLOTS:
        int process( const SyncFile & sf );

        /**
         * Add the (oldUri, newUri) pair to the identifier list
         * @param oldUri has to be a nepomuk:/res/
         * @param newUri can be a nepomuk:/res/ or file:/
         */
        bool identify( int id, const QString & oldUri, const QString & newUri );

        bool ignore(int id, const QString& url, bool ignoreSubDirectories);

    protected:
        virtual void run();

    private :
        QQueue<SyncFileIdentifier *> m_queue;
        QMutex m_queueMutex;
        QWaitCondition m_queueWaiter;

        bool m_stopped;

        QHash< int, SyncFileIdentifier *> m_processes;
        QMutex m_processMutex;

        void emitNotIdentified( int id, const QList<Soprano::Statement> & stList );
    };

}
#endif // IDENTIFICATIONTHREAD_H
