/*
   Copyright (c) 2006-2010 Sebastian Trueg <trueg@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License or (at your option) version 3 or any later version
   accepted by the membership of KDE e.V. (or its successor approved
   by the membership of KDE e.V.), which shall act as a proxy
   defined in Section 14 of version 3 of the license.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "mainwindow.h"
#include "searchhitmodel.h"
#include <nepomuk/queryserviceclient.h>

#include <nepomuk/resourcemanager.h>

#include <KDebug>
#include <KRun>
#include <KComboBox>

#include <QtCore/QCoreApplication>
#include <QtCore/QRegExp>
#include <QtGui/QAbstractItemView>
#include <QtGui/QSortFilterProxyModel>
#include <QtGui/QStatusBar>

Q_DECLARE_METATYPE( Nepomuk::Query::Result )

using namespace Soprano;


MainWindow::MainWindow()
  : QMainWindow(),
    Ui::MainWindow()
{
    setupUi( this );

    Nepomuk::ResourceManager::instance()->init();

    m_searchClient = new Nepomuk::Query::QueryServiceClient( this );

    m_searchCompletion = m_editSearchText->completionObject();

    connect( m_buttonSearch, SIGNAL( clicked() ),
             this, SLOT( slotSearch() ) );
    connect( m_editSearchText, SIGNAL( returnPressed() ),
             this, SLOT( slotSearch() ) );
    connect( m_searchClient, SIGNAL( newEntries( const QList<Nepomuk::Query::Result>& ) ),
             m_resultsView, SLOT( updateResources( const QList<Nepomuk::Query::Result>& ) ) );
    connect( m_searchClient, SIGNAL( totalCount(int) ),
             this, SLOT( slotTotalCount(int) ) );
    connect( m_searchClient, SIGNAL( finishedListing() ),
             this, SLOT( slotFinished() ) );
    connect( m_resultsView, SIGNAL( resourceActivated( const QUrl& ) ),
             this, SLOT( slotResourceActivated( const QUrl& ) ) );
}


MainWindow::~MainWindow()
{
}


void MainWindow::slotSearch()
{
    QString searchText = m_editSearchText->text();

    m_resultsView->clear();

    if ( !searchText.isEmpty() ) {
        m_searchCompletion->addItem( searchText );

        kDebug() << "Searching for" << searchText;

        m_searchClient->desktopQuery( searchText );
    }
}


void MainWindow::slotFinished()
{
    kDebug() << "Search done";
}


void MainWindow::slotTotalCount( int count )
{
    statusBar()->showMessage( i18n("Total result count: %1", count ) );
}


void MainWindow::slotResourceActivated( const QUrl& uri )
{
    if ( uri.isValid() ) {
        // FIXME: Use a plugin-system like the painting one for execution of resources (maybe integrate it into the gui item)
        new KRun( uri, this );
    }
}

#include "mainwindow.moc"
