/*
Copyright (c) 2010 Oszkar Ambrus <aoszkar@gmail.com>
Copyright (C) 2010 Sebastian Trueg <trueg@kde.org>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef RESOURCELISTMODEL_H
#define RESOURCELISTMODEL_H

#include "resourcemodel.h"

#include "nepomuksearchgui_export.h"

namespace Nepomuk
{
namespace Query
{
class Result;
}


    /**
     * A Model for resource items. It provides the label for displaying and the URI for editing of a recourse.
     */

    class NEPOMUKSEARCHGUI_EXPORT ResourceListModel : public Utils::ResourceModel
    {
    public:
        /**
         * Creates a new ResourceListModel that doesn't automatically create an editor
         */
        ResourceListModel(QObject * parent = 0);
        ~ResourceListModel();

        QModelIndex indexForResource( const Resource& res ) const;
        Resource resourceForIndex( const QModelIndex& index ) const;

        /**
         * Sets the list of underlying resources for the model to @p resources
         */
        void setResources(const QList<Nepomuk::Query::Result>& resources);

        /**
         * @reimp
         *
         * @return the resource's label for Qt::DisplayRole and the resource itself for Qt::EditRole,
         * an invalid QVariant() for other roles or for an invalid index
         */
        QVariant data(const QModelIndex& index, int role) const;

        /**
         * @reimp
         */
        int rowCount(const QModelIndex& parent = QModelIndex()) const;

        /**
         * @reimp
         */
        QModelIndex index(int row, int column, const QModelIndex& parent = QModelIndex()) const;

        /**
         * @reimp
         */
        bool removeRows ( int row, int count, const QModelIndex & parent = QModelIndex() );

    private:
        class Private;
        Private * const d;
    };

}

#endif // RESOURCELISTMODEL_H
