/*
 * search_widget_testview.cpp
 *
 * Copyright (C) 2008 Adam Kidder <thekidder@gmail.com>
 */
#include "savesearchdialog.h"
#include "search_widget_testview.h"

#include <KDebug>
#include <KDialog>
#include <KDirLister>
#include <KDirModel>
#include <KLocale>
#include <KUrl>

#include <QtGui/QLabel>

search_widget_testView::search_widget_testView(QWidget *) :
    savedSearches("kionepomukuserqueriesrc")
{
    ui_search_widget_testview_base.setupUi(this);
    model = new KDirModel;

    ui_search_widget_testview_base.listView->setModel( model );

    connect(ui_search_widget_testview_base.searchBox, SIGNAL(textChanged(const QString&)), this, SLOT(searchStringChanged(const QString&)) );
    connect(ui_search_widget_testview_base.searchPane, SIGNAL(queryChanged(QString)), this, SLOT(advancedSearchChanged(const QString&)) );
    connect(ui_search_widget_testview_base.searchButton, SIGNAL(clicked()), this, SLOT(activateSearch()) );
    connect(ui_search_widget_testview_base.searchBox, SIGNAL(returnPressed()), this, SLOT(activateSearch()) );
    connect(ui_search_widget_testview_base.searchPane, SIGNAL(queryChanged(QString)), ui_search_widget_testview_base.searchString, SLOT(setText(QString)));
    connect(ui_search_widget_testview_base.saveSearch, SIGNAL(clicked()), this, SLOT(saveSearch())); 

    ui_search_widget_testview_base.searchBox->setText( "hasTag:nepomuk" );

    activateSearch();

    setAutoFillBackground(true);
}

search_widget_testView::~search_widget_testView()
{

}

void search_widget_testView::activateSearch()
{
    QString query = getQueryString();
    QUrl searchURL( QString( "nepomuksearch:/?query=%1" ).arg(query) );
    kDebug() << "opening search" << searchURL.toString();
    model->dirLister()->openUrl( searchURL );
}

QString search_widget_testView::getQueryString()
{
    return (QString("%1 %2").arg(search).arg(advanced) ).trimmed();
}

void search_widget_testView::saveSearch()
{
    kDebug() << "Saving" << getQueryString();
    KDialog* save = new KDialog(this);

    save->setCaption( "Save this search" );
    save->setButtons( KDialog::Ok | KDialog::Cancel );

    SaveSearchDialog* dialog = new SaveSearchDialog( getQueryString() );

    save->setMainWidget( dialog );
    save->setModal( true );

    connect(dialog, SIGNAL(nameChanged(QString)), this, SLOT(setName(QString)));
    connect(save, SIGNAL(okClicked()), this, SLOT(doSaveSearch()));

    save->show();

    //SaveSearchDialog* dialog = new SaveSearchDialog(this);
}

void search_widget_testView::setName(const QString& name)
{
    searchName = name;
}


void search_widget_testView::doSaveSearch()
{
    QString query = getQueryString();

    KConfigGroup searches = savedSearches.group("Searches");
    QStringList currentSearches = searches.readEntry("All searches", QStringList() );
    //TODO: look for save search name before saving
    kDebug() << "saving" << searchName;
    currentSearches.append( searchName );
    searches.writeEntry( "All searches", currentSearches);

    KConfigGroup newGroup = savedSearches.group( searchName );
    newGroup.writeEntry("Query", QString("\"?query=%1\"").arg(query));

    savedSearches.sync(); 
}

void search_widget_testView::loadSavedSearches()
{
}

void search_widget_testView::saveSavedSearches()
{
}

void search_widget_testView::searchStringChanged(const QString& searchString)
{
    search = searchString;
}

void search_widget_testView::advancedSearchChanged(const QString& advancedString)
{
    advanced = advancedString;
}

#include "search_widget_testview.moc"
