/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include <iostream>

#include <QString>
#include <QUrl>

#include <Soprano/Model>
#include <Soprano/Node>
#include <Soprano/QueryResultIterator>
#include <Soprano/Vocabulary/NAO>

#include <KAboutData>
#include <KApplication>
#include <KCmdLineArgs>

#include <Nepomuk/ResourceManager>

void printNode( const Soprano::Node& node )
{
    QString text;

    switch ( node.type() ) {
    case Soprano::Node::ResourceNode:
        text = node.uri().toString();
        break;
    case Soprano::Node::LiteralNode:
        text = node.literal().toString();
        break;
    case Soprano::Node::BlankNode:
        text = node.identifier();
    default:
        ;
    }

    std::cout << text.toUtf8().constData() << std::endl;
}

int main( int argc, char* argv[] )
{
    KAboutData aboutData( QByteArray( "ListOntologies" ), QByteArray(), KLocalizedString(), QByteArray() );
    KCmdLineArgs::init( argc, argv, &aboutData );
    KApplication app;

    Soprano::Model* mainModel = Nepomuk::ResourceManager::instance()->mainModel();
    const QString query = QString( "select ?ns "
                                   "where {"
                                   "    ?g <%1> ?ns"
                                   "}" )
                          .arg( Soprano::Vocabulary::NAO::hasDefaultNamespace().toString() );

    Soprano::QueryResultIterator it = mainModel->executeQuery( query, Soprano::Query::QueryLanguageSparql );
    while ( it.next() ) {
        printNode( it.binding( "ns" ) );
    }
    std::cout << std::endl;
    return 0;
}
