/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include <iostream>

#include <QString>
#include <QUrl>

#include <Soprano/Model>
#include <Soprano/Node>
#include <Soprano/QueryResultIterator>
#include <Soprano/StatementIterator>
#include <Soprano/Vocabulary/NAO>
#include <Soprano/Vocabulary/XMLSchema>

#include <KAboutData>
#include <KApplication>
#include <KCmdLineArgs>

#include <Nepomuk/ResourceManager>

void printNode( const Soprano::Node& node )
{
    std::cout << node.toString().toUtf8().constData() << std::endl;
}

int main( int argc, char* argv[] )
{
    KAboutData aboutData( QByteArray( "LoadOntology" ), QByteArray(), KLocalizedString(), QByteArray() );
    KCmdLineArgs::init( argc, argv, &aboutData );
    KApplication app;

    Soprano::Model* mainModel = Nepomuk::ResourceManager::instance()->mainModel();
    const QUrl uri( "http://www.semanticdesktop.org/ontologies/2007/08/15/nrl#" );
    const QString query = QString( "construct { ?s ?p ?o }"
                                   "where {"
                                   "    graph ?g { ?s ?p ?o } ."
                                   "    ?g <%1> \"%2\"^^<%3> ."
                                   "}" )
                          .arg( Soprano::Vocabulary::NAO::hasDefaultNamespace().toString() )
                          .arg( QString::fromUtf8( uri.toEncoded() ) )
                          .arg( Soprano::Vocabulary::XMLSchema::string().toString() );

    const Soprano::QueryResultIterator qit = mainModel->executeQuery( query, Soprano::Query::QueryLanguageSparql );
    if ( qit.isGraph() ) {
        Soprano::StatementIterator sit = qit.iterateStatements();
        while ( sit.next() ) {
            printNode( sit.current().subject() );
            printNode( sit.current().predicate() );
            printNode( sit.current().object() );
            std::cout << std::endl;
        }
    }
}
