/*
 * This file is part of Search Project.
 *
 * Copyright (C) 2007 Daniele Galdi <daniele.galdi@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include <QDebug>
#include <QBrush>
#include <QPainter>
#include <QHeaderView>
#include <QModelIndex>
#include <QAbstractItemModel>

#include "ItemWidget.h"
#include "ItemDelegate.h"
#include "StrigiHitsModel.h"

#include "TreeView.h"

TreeView::TreeView( QWidget *parent )
  :QTreeView(parent)
{
  header()->hide();
  //setRootIsDecorated( false );
  setUniformRowHeights( false );
  //setAnimated(true);
  
  setItemDelegate( new ItemDelegate( this ) );

  connect( this,
	   SIGNAL( doubleClicked ( const QModelIndex &) ),
	   this,
	   SLOT( stateChanged( const QModelIndex & ) ) );
}
  
TreeView::~TreeView()
{
}

void TreeView::destroyWidget( const QModelIndex &index )
{
  qDebug() << "####################TreeView::destroyWidget" << index;

  return;

  QWidget* widget = indexWidget( index );
  disconnect( widget, 
	     SIGNAL( stateChanged( const QModelIndex & ) ), 
	     this, 
	     SLOT( destroyWidget( const QModelIndex & ) ) );
  setIndexWidget( index, 0L );

  StrigiHitsModel* m = qobject_cast<StrigiHitsModel*>( model() );
  m->invalidateLayout();
}

void TreeView::stateChanged( const QModelIndex &index )
{
  if ( index.internalId() != 2 ) return;
  if ( indexWidget( index ) ) return;

  return;

  qDebug() << "####################TreeView::stateChanged" << index;

  ExpandedItemWidget* widget = new ExpandedItemWidget( index );
  setIndexWidget( index, widget );
  connect( widget, 
	   SIGNAL( stateChanged( const QModelIndex & ) ), 
	   this, 
	   SLOT( destroyWidget( const QModelIndex & ) ) );

  StrigiHitsModel* m = qobject_cast<StrigiHitsModel*>( model() );
  m->invalidateLayout();
}

void TreeView::drawRow( QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index ) const
{
  QTreeView::drawRow( painter, option, index );
}

void TreeView::drawBranches( QPainter * painter, const QRect & rect, const QModelIndex & index ) const
{
  // Disable branches drawing
  if ( index.internalId() != 1 )
    return;
  
  if ( isExpanded( index ) )
  {
    painter->fillRect( rect, QBrush(Qt::gray) );
  } 

  painter->fillRect( rect, QBrush(Qt::red) );
}

#include "TreeView.moc"
