#!/usr/bin/env python
# -*- coding: utf-8 -*-
"A simple KIO client which lists all results from a KIO dir listing"

import sys

from PyQt4 import QtCore, QtGui
from PyKDE4 import kdecore, kio

def printUDSEntry(uds):
    for field in uds.listFields():
      if field&kio.KIO.UDSEntry.UDS_NUMBER:
        print "%d: %d" % (field, uds.numberValue(field,0))
      else:
        print "%d: %s" % (field, uds.stringValue(field))


class KIOResultListener(object):
    "Listen to the results from a KDirLister and print them"

    def __init__(self, dirlister, connectCompleted = True):
        self.dirLister = dirlister
        self.dirLister.newItems.connect(self.newItems)
        self.dirLister.percent.connect(self.percent)
        self.dirLister.itemsDeleted.connect(self.itemsDeleted)
        self.resultCount = 0
        if connectCompleted:
            self.dirLister.completed.connect(self.listingCompleted)

    def newItems(self, itemList):
        for item in itemList:
            print "New item    : %s (%s)" % (item.text(), item.name())
            printUDSEntry(item.entry())
            self.resultCount += 1

    def itemsDeleted(self,itemList):
        for item in itemList:
            print "Item deleted: %s (%s)" % (item.text(),item.url().url())

    def percent(self,p):
        print "Percent: %d" % p
        
    def listingCompleted(self):
        print "Listing done. %i results." % self.resultCount
        QtCore.QCoreApplication.instance().quit()
        
        
def main():
    # create the application and component necessary for KIO
    app = QtGui.QApplication(sys.argv)
    comp = kdecore.KComponentData("nepomuksearch-kio-client")

    # create the dirlister
    lister = kio.KDirLister()
    
    # create our listener
    listener = KIOResultListener(lister, True)

    # start the dirlister
    lister.openUrl(kdecore.KUrl(sys.argv[1]))

    # and away we go
    app.exec_()
    

if __name__ == '__main__':
    main()
