/*
  Copyright (C) 2010 Michael Leupold <lemma@confuego.org>

  SHA-1 implementation taken from kwalletbackend:
    Copyright (C) 2001 George Staikos <staikos@kde.org>
    Based heavily on SHA1 code from GPG 1.0.3 (C) 1998 FSF

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "hashtest.h"
#include "sha1.h"
extern "C" {
#include "../hash.h"
}

#include "qtest_kde.h"

QTEST_KDEMAIN_CORE(HashTest)

// legacy function from KWalletBackend
static int legacypassword2hash(const QByteArray& password, QByteArray& hash) {
   SHA1 sha;
   int shasz = sha.size() / 8;

   assert(shasz >= 20);

   QByteArray block1(shasz, 0);
   sha.process(password.data(), qMin(password.size(), 16));

   // To make brute force take longer
   for (int i = 0; i < 2000; i++) {
      memcpy(block1.data(), sha.hash(), shasz);
      sha.reset();
      sha.process(block1.data(), shasz);
   }

   sha.reset();

   if (password.size() > 16) {
      sha.process(password.data() + 16, qMin(password.size() - 16, 16));
      QByteArray block2(shasz, 0);
      // To make brute force take longer
      for (int i = 0; i < 2000; i++) {
         memcpy(block2.data(), sha.hash(), shasz);
         sha.reset();
         sha.process(block2.data(), shasz);
      }

      sha.reset();

      if (password.size() > 32) {
         sha.process(password.data() + 32, qMin(password.size() - 32, 16));

         QByteArray block3(shasz, 0);
         // To make brute force take longer
         for (int i = 0; i < 2000; i++) {
            memcpy(block3.data(), sha.hash(), shasz);
            sha.reset();
            sha.process(block3.data(), shasz);
         }

         sha.reset();

         if (password.size() > 48) {
            sha.process(password.data() + 48, password.size() - 48);

            QByteArray block4(shasz, 0);
            // To make brute force take longer
            for (int i = 0; i < 2000; i++) {
               memcpy(block4.data(), sha.hash(), shasz);
               sha.reset();
               sha.process(block4.data(), shasz);
            }

            sha.reset();
            // split 14/14/14/14
            hash.resize(56);
            memcpy(hash.data(),      block1.data(), 14);
            memcpy(hash.data() + 14, block2.data(), 14);
            memcpy(hash.data() + 28, block3.data(), 14);
            memcpy(hash.data() + 42, block4.data(), 14);
            block4.fill(0);
         } else {
            // split 20/20/16
            hash.resize(56);
            memcpy(hash.data(),      block1.data(), 20);
            memcpy(hash.data() + 20, block2.data(), 20);
            memcpy(hash.data() + 40, block3.data(), 16);
         }
         block3.fill(0);
      } else {
         // split 20/20
         hash.resize(40);
         memcpy(hash.data(),      block1.data(), 20);
         memcpy(hash.data() + 20, block2.data(), 20);
      }
      block2.fill(0);
   } else {
      // entirely block1
      hash.resize(20);
      memcpy(hash.data(), block1.data(), 20);
   }

   block1.fill(0);

   return 0;
}

static QByteArray randomByteArray(int length)
{
   QByteArray arr(length, 0);
   for (int i = 0; i < length; ++i) {
      arr[i] = qrand() % 128;
   }
   return arr;
}

void HashTest::initTestCase()
{
   qsrand(time(0));
}

void HashTest::hashTest()
{
   for (int i = 0; i < 256; ++i) {
      if (i == 103 || i == 167 || i == 231) {
         QEXPECT_FAIL("", "Hashing for the following passwords lengths doesn't work: 103, 167, 231", Continue);
      }

      unsigned char newhash[56];
      int newhashsz;

      QByteArray input = randomByteArray(i);
      password_to_hash(newhash, &newhashsz, (unsigned char*)input.constData(), input.size());
      QByteArray baNewHash;
      baNewHash.resize(newhashsz);
      memcpy(baNewHash.data(), newhash, newhashsz);
      QByteArray oldHash;
      legacypassword2hash(input, oldHash);

      QVERIFY2(baNewHash == oldHash,
               QString("Mismatch at password length %1").arg(i).toLatin1().constData());
   }
}

void HashTest::cleanupTestCase()
{
}

#include "hashtest.moc"
