/***************************************************************************
 *   Copyright 2008 by Thomas Schildknecht <danakil.kde@gmail.com>         *
 *   Copyright 2010 by Stefan Majewsky <majewsky@gmx.net>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

#ifndef AWC_WINDOWMANAGER_H
#define AWC_WINDOWMANAGER_H

#include <Plasma/DataEngine>

namespace Awc
{
    enum WindowStateFlag
    {
        IsShaded = 1 << 0,
        IsKeptAbove = 1 << 1,
        IsKeptBelow = 1 << 2,
        IsMaximized = 1 << 3,
        IsMinimized = 1 << 4,
        IsTemporarilyRestored = 1 << 5 ///< When cycling through windows, we restore minimized windows when they are activated. These should be minimized again when the user continues cycling, so we need to flag these windows.
    };
    Q_DECLARE_FLAGS(WindowState, WindowStateFlag)

    enum CyclingStrategyFlag
    {
        CyclingEnabled = 1 << 0,
        CycleOnAllDesktops = 1 << 1, ///< If not set, only windows on the current desktop will be considered.
        IncludeMinimizedWindows = 1 << 2
    };
    Q_DECLARE_FLAGS(CyclingStrategy, CyclingStrategyFlag)

    class WindowManager : public QObject
    {
        Q_OBJECT
        public:
            explicit WindowManager(Plasma::DataEngine* tasksEngine, QObject* parent = 0);

            Awc::CyclingStrategy cyclingStrategy() const;
            void setCyclingStrategy(Awc::CyclingStrategy strategy);
            void setReactivateActiveWindowAfterOperations(bool reactivate);
        Q_SIGNALS:
            void shadedChanged(bool shaded);
            void keptAboveChanged(bool keptAbove);
            void keptBelowChanged(bool keptBelow);
            void maximizedChanged(bool maximized);
            void activeWindowChanged(bool activeWindowAvailable);
        public Q_SLOTS:
            void publishActiveWindowState(); ///< May be triggered manually to propagate state information to newly connected observers.
            void triggerActiveWindowAction(const QString& actionName);
            void cycleToAdjacentWindow(int direction);
        protected:
            void executeWindowAction(WId wid, const QString& actionName);
        protected Q_SLOTS:
            void dataUpdated(const QString& name, const Plasma::DataEngine::Data& data);
            void sourceAdded(const QString& source);
            void sourceRemoved(const QString& source);
        private:
            Awc::CyclingStrategy m_cyclingStrategy;
            bool m_reactivateActiveWindowAfterOperations;

            Plasma::DataEngine* m_tasksEngine;
            WId m_activeWindow;
            int m_activeDesktop; //defined as the desktop of the m_activeWindow, with the exception that m_activeDesktop won't be changed from the previous value if the desktop is -1 (which means that the window is on all desktops)
            QMap<WId, Awc::WindowState> m_windowStates;
            QMap<int, QList<WId> > m_windowOrder; //key = desktop
    };
}

Q_DECLARE_OPERATORS_FOR_FLAGS(Awc::WindowState)
Q_DECLARE_OPERATORS_FOR_FLAGS(Awc::CyclingStrategy)

#endif //AWC_WINDOWMANAGER_H
