/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Copyright (C) 2008 by Dennis Nienhüser <earthwings@gentoo.org>        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef CELL_FACTORY_H_
#define CELL_FACTORY_H_

#include <QtCore/QObject>
#include <QtCore/QString>
#include <QtCore/QVariant>

// Forward declarations
class AbstractCell;
class QGraphicsWidget;

/**
 * Factory able to construct and setup cells for known identifiers.
 *
 */
class CellFactory: public QObject
{
Q_OBJECT

public:
	/**
	 * Construct and setup a cell for the given identifier
	 * @param id Identifier, must be among those returned by #knownCells
	 * @param properties Contact's properties used to setup the cell
	 * @param parent Parent widget
	 * @return A cell ready to use. If the identifier is unknown, a simple label will be returned.
	 */
	static AbstractCell *construct(const QString &id, const QVariantMap &properties, QGraphicsWidget *parent);

	/**
	 * @return A suggestion about a sane default cell setup
	 */
	static QStringList defaultCells();

	/**
	 * @return A list of known cells that can be used in #construct
	 */
	static QStringList knownCells();

};

#endif // CELL_FACTORY_H_
