/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Copyright (C) 2008 by Dennis Nienhüser <earthwings@gentoo.org>        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "iconcell.h"

#include <QtGui/QGraphicsLinearLayout>

#include "kdebug.h"
#include "kicon.h"

IconCell::IconCell(const QString id, const QString &fallbackIcon,
		const QVariantMap &properties, QGraphicsWidget *parent) :
	AbstractCell(parent), m_id(id), m_fallbackIcon(fallbackIcon)
{
	m_icon = new Plasma::IconWidget();
	m_icon->setMinimumSize(QSizeF(32, 32));

	QGraphicsLinearLayout *layout = new QGraphicsLinearLayout(Qt::Horizontal);
	setLayout(layout);
	layout->addItem(m_icon);

	propertiesChanged(properties);
}

IconCell::~IconCell()
{
	m_icon->deleteLater();
}

void IconCell::propertiesChanged(const QVariantMap &properties)
{
	// TODO: compare to old state and animate
	QString pathOrName = properties[m_id].toString();
	if (pathOrName.isEmpty())
	{
		m_icon->setIcon(KIcon(m_fallbackIcon));
	}
	else
	{
		m_icon->setIcon(pathOrName);
	}
}
