/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Copyright (C) 2008 by Dennis Nienhüser <earthwings@gentoo.org>        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "labelcell.h"

#include "kdebug.h"

#include <QtGui/QGraphicsLinearLayout>
#include <QtCore/QTimer>

LabelCell::LabelCell(const QString id, const QVariantMap &properties,
		QGraphicsWidget *parent) :
	AbstractCell(parent), m_id(id), m_animate(false)
{
	m_label = new Plasma::Label();
	m_label->setMinimumWidth(150);
	m_label->setMaximumWidth(150);

	QGraphicsLinearLayout *layout = new QGraphicsLinearLayout(Qt::Horizontal);
	setLayout(layout);
	layout->addItem(m_label);

	propertiesChanged(properties);
}

LabelCell::~LabelCell()
{
	// nothing to do
}

void LabelCell::propertiesChanged(const QVariantMap &properties)
{
	const QString text = properties[m_id].toString();
	if (text != m_text)
	{
		setText(text);
	}
}


void LabelCell::setAnimationsEnabled(bool enabled)
{
	m_animate = enabled;
}

void LabelCell::setText(const QString &text)
{
	if (!m_animate)
	{
		m_text = text;
		m_label->setText(text);
		return;
	}

	if (text == m_text)
	{
		return;
	}

	m_text = text;
	m_subText = "";
	animatedTextChange();
}


void LabelCell::animatedTextChange()
{
	if (m_subText.size() >= m_text.size())
	{
		m_label->setText(m_text);
		return;
	}

	m_subText = m_text.left(m_subText.length()+1);
	m_label->setText(m_subText);
	QTimer::singleShot( 150, this, SLOT(animatedTextChange()) );
}
