/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Copyright (C) 2008 by Dennis Nienhüser <earthwings@gentoo.org>        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef LINE_EDIT_CELL_H_
#define LINE_EDIT_CELL_H_

#include "abstractcell.h"

#include <Plasma/LineEdit>

// Forward declarations
class QGraphicsLinearLayout;

/**
 * Uses a Plasma::LineEdit for input.
 * @todo FIXME: This is not as generic as the name suggests. Change to ChatInput or similar.
 */
class LineEditCell: public AbstractCell
{
Q_OBJECT

public:
	/**
	 * Constructor
	 * @param id Property key identifying the message receiver
	 * @param properties Contacts properties
	 * @param parent Parent widget
	 */
	LineEditCell(const QString receiverId, const QString reachableId, const QVariantMap &properties, QGraphicsWidget *parent)				;

	/** Destructor */
	virtual~ LineEditCell();

	/**
	 * Reimplemented from AbstractCell
	 */
	virtual void propertiesChanged(const QVariantMap &properties);

	Q_SIGNALS:
	/** Emitted when return is pressed and the input is non-empty
	 * @param receiver Message receiver
	 * @param text Message (text input)
	 * */
	void inputFinished(QString receiver, QString text);

private Q_SLOTS:
	/** Check whether signal needs to be emitted and clear text */
	void inputFinished();

private:
	/** Property key identifying the message receiver */
	QString m_receiverId;

	/** Property key identifying contact reachability */
	QString m_reachableId;

	/** Message receiver */
	QString m_receiver;

	/** Line edit used to gather input */
	Plasma::LineEdit *m_lineEdit;

	/** Widget layout */
	QGraphicsLinearLayout *m_layout;
};

#endif // LINE_EDIT_CELL_H_
