/*
 *   Copyright (C) 2008 Dennis Jansen <Dennis.Jansen@web.de>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 or later
 *   as published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/* Description:
 * plasma-darkstat is a plasmoid that (for now) presents darkstat's xml data
 *
 */

#include "plasma-darkstat.h"
#include <QPainter>
#include <QFontMetrics>

#include <QGraphicsGridLayout>

#include <Plasma/Theme>
#include <Plasma/Label>

#include <KDebug>
#include <KConfigDialog>

bool VERBOSE=false; // set to true for more verbose output.

PlasmaDarkstat::PlasmaDarkstat(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args)
{
   setBackgroundHints(DefaultBackground);
   setAspectRatioMode(Plasma::IgnoreAspectRatio);
   setMinimumSize(60, 40); // replace with correct numbers
   resize(100, 100); // normal
}

PlasmaDarkstat::~PlasmaDarkstat()
{
    if (hasFailedToLaunch()) {
        // Do some cleanup here
    } else {
        // Save settings
    }
    UpdateLoopTimer->stop();
    delete UpdateLoopTimer;
}

void PlasmaDarkstat::init()
{

   setHasConfigurationInterface(true);


   /* initialize labels*/
   m_TextKBpsi = new Plasma::Label(this);
   m_TextKBpso = new Plasma::Label(this);
   m_TextKB = new Plasma::Label(this);
   //m_TextPackets = new Plasma::Label(this);

   /* basic vertical layout with horizontals inside */
   m_layoutv_main = new QGraphicsLinearLayout(Qt::Vertical);
   m_layouth_misc = new QGraphicsLinearLayout(Qt::Vertical);
   m_layouth_IO = new QGraphicsLinearLayout;

   /* bundle in and out horizontally */
   m_layouth_IO->addItem(m_TextKBpsi);
   m_layouth_IO->addItem(m_TextKBpso);
   m_layoutv_main->addItem(m_layouth_IO);
   /* KB vertically */
   m_layoutv_main->addItem(m_TextKB);

   //m_layoutv_main->setContentsMargins(0, 0, 0, 0);
   m_layoutv_main->setSpacing(0);

   /* Put Packets into extra layout to achieve event spacing */
   //m_layouth_misc->addItem(m_TextPackets);
   //m_layoutv_main->addItem(m_layouth_misc); // who really cares about the packets?

   m_TextKBpsi->setText( i18n("Please Wait..."));

   setLayout(m_layoutv_main);
   /* FIXME: look pretty */

   job = 0;

   /* FIXME: save and restore configuration */
   url = "http://127.0.0.1:667/graphs.xml";
   Download_init(); /* initialize http settings */
   Download(); /* first download */

    /* init Update Loop Timer and its helper variables */
   UpdateLoopTimer = new QTimer(this);
   UpdateLoopTimer->setSingleShot(true); /* turn off auto-repeat */
   UpdateLoopInterval = 1000; /* set timer interval in ms */
   KeepGoing = true;
   connect(UpdateLoopTimer, SIGNAL(timeout()), this, SLOT(Download()));
}

void PlasmaDarkstat::constraintsUpdated(Plasma::Constraints constraints)
{
   Q_UNUSED(constraints);
    // FIXME
}



void PlasmaDarkstat::createConfigurationInterface(KConfigDialog *dialog)
{ /* Thanks: Luna - WindowsUninstall, jackrabbit, aseigo */
    QWidget *widget = new QWidget();
    ui.setupUi(widget);
    ui.timerSpinBox->setSuffix(ki18np(" second", " seconds"));
/* from example in http://api.kde.org/4.x-api/kdelibs-apidocs/kdeui/html/classKDialog.html */
    dialog->setMainWidget( widget );
    dialog->setButtons(KDialog::Ok | KDialog::Cancel ); // | KDialog::Apply
    connect(dialog, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
    connect(dialog, SIGNAL(okClicked()), this, SLOT(configAccepted()));

   dialog->addPage(widget, dialog->windowTitle(), icon());
   /* It only works once, may as well use OK */
   //dialog->showButton(KDialog::Apply, false);
    /* FIXME: Reenable Apply button on input change */
   // connect( widget, SIGNAL( changed( bool ) ), dialog, SLOT( enableButtonApply( bool ) ) );
   // connect(parent, SIGNAL(applyClicked()), parent, SLOT(enableButton(KDialog::Apply, true)));
   // connect( http, SIGNAL( done( bool ) ), parent, SLOT( enableButtonApply( bool ) ) );
   //dialog->enableButtonApply( false );
}

void PlasmaDarkstat::configAccepted()
{
   QString host = ui.hostLineEdit->text();
   int port = ui.portSpinBox->value();
   QString file = '/' + ui.fileLineEdit->text();

   url.setHost(host);
   url.setPort(port);
   url.setPath(file);

   UpdateLoopInterval = 1000 * ui.timerSpinBox->value(); /* set timer interval in ms */
   KeepGoing = ui.updateCheckBox->isChecked();
   if (job) {
       job->kill();
       job = 0;
   }
   Download_init();
   Download();
}

void PlasmaDarkstat::Download_init()
{
   verboseLog("Download_init()");

   if (job) {
       job->kill();
       job = 0;
   }
}

void PlasmaDarkstat::Download()
{
   verboseLog("Download()");

   /* get the file and store in QByteArray */
   job = KIO::storedGet(url);
   connect(job, SIGNAL(finished(KJob*)), SLOT(DownloadFinished()));
}

void PlasmaDarkstat::DownloadFinished()
{
   verboseLog("DownloadFinished()");
   if (job->error())
   {
      err ("http-error:" + job->errorString());
      UpdateLoopInterval+=1000;
   }
   else
   {
      graphs_xml=job->data();
      job = 0;
      UpdateText();
      update();
      Repeat();
      /* calling repeat() here ensures the UpdateLoopInterval
      is started after the previous download is finished and processed */
   }
}

QString ElementText(QString input, QString element, int start = 1)
{
   int begin = -1, end = -1;

   begin = input.indexOf(element, start); /* find element */

   if (begin >= 0)
   {
      begin = input.indexOf('"', begin) + 1; /* after starting parenthesis */
      end = input.indexOf('"', begin) - 1; /* before ending parenthesis */
   }

   if (VERBOSE)
      kDebug()    << element << "starts at " << begin << " and ends at "
            << end << " length " << ( end - begin ) +1;

   if (begin <= end)
      return input.mid(begin, (end - begin) +1 );
   else
      // return "Not found: \"" + element + "\"";
      return QString();
}

QString GraphValue(QString *input, QString section, QString element, int position, int start = 1)
{
   int begin = -1;

   begin = input->indexOf(section, start); /* find section */
   if(VERBOSE) kDebug() << "section " << section << " begin " << begin;
   QString second; // name second
   second.setNum(position);
   begin = input->indexOf("p=\""+ second + "\"", begin); /* find position */
   if(VERBOSE) kDebug() << "p=" + second << "is at " << begin;
   begin = input->indexOf(element, begin); /* find element */
   if(VERBOSE) kDebug() << "element " << element << " begin " << begin;

   return ElementText(*input, element, begin);
}

/* Format Strings as B, KiB, MB, ... */
QString toFormat(QString n)
{
   /* don't return the KGlobal directly for a clean exit, avoid segfault */
   /* FIXME: The segfault still sometimes happens */
   n.remove(','); // remove preformatting , from older darkstat's xml.
   QString tmp=KGlobal::locale()->formatByteSize( n.toDouble());
   return  tmp;
}

void PlasmaDarkstat::UpdateText()
{
   time=QTime::currentTime();
   /* Give me the GraphValue of last second */
   QString bps=GraphValue(&graphs_xml, "seconds", "i=", ((60+(time.second()-1))%60));
   QString obps=GraphValue(&graphs_xml, "seconds", "o=", ((60+(time.second()-1))%60));
   QString pkgs=ElementText(graphs_xml, "tp=");
   QString bs=ElementText(graphs_xml, "tb=");

   m_TextKBpsi->setText( toFormat(bps));
   m_TextKBpso->setText( toFormat(obps));
   m_TextKB->setText( toFormat( bs ) );
   //m_TextPackets->setText( toFormat( pkgs ) + " pks");
}

void PlasmaDarkstat::Repeat()
{
   if (KeepGoing)
      UpdateLoopTimer->start(UpdateLoopInterval);
      /* Download() is called on timeout() SIGNAL */
   else
      UpdateLoopTimer->stop();

      if (VERBOSE)
         kDebug() << "SingleShot Timer: " << UpdateLoopTimer->isSingleShot();
}


void verboseLog(QString text)
{
   if (VERBOSE)
   kDebug() << "Debug: " << text;
}

void PlasmaDarkstat::err(QString text)
{
   m_TextKB->setText(text);
}

#include "plasma-darkstat.moc"
