/***************************************************************************
 *   Copyright (C) 2010 by Matthieu Gallien <matthieu_gallien@yahoo.fr>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef TASKS_VIEW_HEADER
#define TASKS_VIEW_HEADER

#include <QGraphicsWidget>
#include <QModelIndex>

class QGraphicsLinearLayout;
class QAbstractItemModel;
class TasksViewItem;
class DataEnginesTasks;

namespace Plasma
{
    class DataEngine;
}

class TasksView : public QGraphicsWidget
{
    Q_OBJECT
public:

    TasksView(DataEnginesTasks *applet, Plasma::DataEngine *tasksEngine,
              Plasma::DataEngine *statusNotifierItemEngine,
              QGraphicsItem *parent = 0, Qt::WindowFlags wFlags = 0);

    QAbstractItemModel* model() const;
    void setModel(QAbstractItemModel *model);
    void setOrientation(Qt::Orientation orientation);

protected slots:

    void sourceDataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight);
    void sourceModelReset();
    void sourceRowsInserted(const QModelIndex &parent, int start, int end);
    void sourceRowsAboutToBeRemoved(const QModelIndex &parent, int start, int end);

protected:

    void resetView();

    void insertNewItemInView(const QModelIndex &modelIndex, int viewPosition);

    void sourceRowInserted(const QModelIndex &newItem, const QModelIndex &parent);

    void sourceRowAboutToBeRemoved(const QModelIndex &newItem, const QModelIndex &parent);

private:

    void debugPrint() const;

    DataEnginesTasks *m_applet;
    Plasma::DataEngine *m_tasksEngine;
    Plasma::DataEngine *m_statusNotifierItemEngine;
    QGraphicsLinearLayout *m_layout;
    QAbstractItemModel *m_model;
    QList<TasksViewItem*> m_items;
};

#endif
