/*
 *   Copyright (C) 2007 Ivan Cukic <ivan.cukic+kde@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser/Library General Public License version 2,
 *   or (at your option) any later version, as published by the Free
 *   Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser/Library General Public License for more details
 *
 *   You should have received a copy of the GNU Lesser/Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "embedded_titlebar.h"
#include <KDebug>
#include <QPainter>
#include <QFontMetrics>
#include <QSizeF>

#include <plasma/containment.h>
#include <plasma/panelsvg.h>
#include <plasma/theme.h>
#include <plasma/animator.h>

#include <climits>

EmbeddedTitlebarApplet::EmbeddedTitlebarApplet(QObject * parent, const QVariantList & args):
    Plasma::Applet(parent, args)
{
    setHasConfigurationInterface(false);
    // setBackgroundHints(NoBackground);
    setAspectRatioMode(Plasma::IgnoreAspectRatio);
    resize(500, 58);
}

EmbeddedTitlebarApplet::~EmbeddedTitlebarApplet()
{
}

void EmbeddedTitlebarApplet::init()
{
    setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));
    setMaximumSize(INT_MAX, INT_MAX);

    connect(TaskManager::TaskManager::self(), SIGNAL(taskAdded(TaskPtr)),
            this, SLOT(taskAdded(TaskPtr)));
    connect(TaskManager::TaskManager::self(), SIGNAL(taskRemoved(TaskPtr)),
            this, SLOT(taskRemoved(TaskPtr)));

    QMapIterator<WId,TaskPtr> iter(TaskManager::TaskManager::self()->tasks());
    while (iter.hasNext()) {
        iter.next();
        taskAdded(iter.value());
    }

}

void EmbeddedTitlebarApplet::taskAdded(TaskPtr task)
{
    connect(
            task.constData(), SIGNAL(activated()),
            this, SLOT(taskActivated())
    );
}

void EmbeddedTitlebarApplet::taskRemoved(TaskPtr task)
{
    disconnect(
            task.constData(), SIGNAL(activated()),
            this, SLOT(taskActivated())
    );
}

void EmbeddedTitlebarApplet::taskActivated()
{
    Task * task = (Task *)sender();
    m_title = task->name();
    update();
}

void EmbeddedTitlebarApplet::constraintsEvent(Plasma::Constraints constraints)
{
}

void EmbeddedTitlebarApplet::paintInterface(QPainter * p, const QStyleOptionGraphicsItem * option, const QRect & contentsRect)
{
    p->setPen(QPen(Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor), 1.0));
    QRectF widgetRect = QRectF(0, 0, size().width(), size().height());
    QRectF titleRect  = p->boundingRect(widgetRect,
            Qt::AlignCenter | Qt::TextSingleLine, m_title);
    p->drawText(titleRect,
            Qt::AlignCenter | Qt::TextSingleLine | Qt::ElideRight, m_title);
}

void EmbeddedTitlebarApplet::mousePressEvent(QGraphicsSceneMouseEvent * event)
{
}

void EmbeddedTitlebarApplet::mouseReleaseEvent(QGraphicsSceneMouseEvent * event)
{
}

void EmbeddedTitlebarApplet::mouseMoveEvent(QGraphicsSceneMouseEvent * event)
{
}

#include <embedded_titlebar.moc>
