/*
 *   Copyright (C) 2008 Schutz Sacha <istdasklar@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License version 2,
 *   or (at your option) any later version, as published by the Free
 *   Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.*/// Here we avoid loading the header multiple times


#ifndef KFORTUNOID_HEADER
#define KFORTUNOID_HEADER

#include <KIcon>

#include <Plasma/Applet>

#include <QColor>
#include <QFont>

namespace Plasma
{
    class Label;
    class ToolButton;
    class Frame;
}

class QSizeF;
class FortunoidSettings;

// Define our plasma Applet
class KFortunoid : public Plasma::Applet
{
    Q_OBJECT
    public:
        // Basic Create/Destroy
        KFortunoid(QObject *parent, const QVariantList &args);
        ~KFortunoid();

        // The paintInterface procedure paints the applet to screen
        void paintInterface(QPainter *painter,
                const QStyleOptionGraphicsItem *option,
                const QRect& contentsRect);
        void init();

        void constraintsEvent(Plasma::Constraints constraints);
        void createConfigurationInterface(KConfigDialog *parent);

        QString getNewQuote();

    public slots:
       void slotPrevQuote();
       void slotNextQuote();
       void configAccepted();
       void refresh();
       void readData();
       void writeData();

    protected:
       void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
       void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);

    private:
        // ui items
        Plasma::Label * m_labelQuote;
        Plasma::ToolButton * m_backButton;
        Plasma::ToolButton * m_nextButton;
        Plasma::Frame * m_controlFrame;
        FortunoidSettings * m_settings;
        
        // data
        int m_currentQuote;
        QVector<QString> m_quotes;
        int m_history;
        QTimer * m_timer;
        
        // settings values
        double m_delay;
        QString m_arguments;
        QColor m_color;
        QFont m_font;
        // font to use when in panel
        QFont m_panelFont;
        // additional arguments to use when in panel
        QString m_panelArguments;
        QString m_pathFortune;
};

// This is the command that links your applet to the .desktop file
K_EXPORT_PLASMA_APPLET(fortunoid, KFortunoid)
#endif
