/***************************************************************************
 *   Copyright © 2009 Reza Fatahilah Shah <rshah0385@kireihana.com>        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/
#include "controlwidget.h"
#include <KDebug>
#include <QPainter>
#include <QTimer>
#include <QStyleOptionGraphicsItem>
#include <QGraphicsSceneMouseEvent>
#include <Plasma/Svg>
#include <Plasma/Theme>

ControlWidget::ControlWidget(QGraphicsItem *parent, Qt::WindowFlags wFlags)
        : QGraphicsWidget(parent, wFlags), mPictureButtonRect(QPoint(0, 10), QPoint(36, 42)),
        mVideoButtonRect(QPoint(86, 10), QPoint(126, 42)), mCircleButtonRect(QPoint(36, 0), QPoint(84, 50))
{
    setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    setMinimumSize(126, 50);

    QString svgFile = Plasma::Theme::defaultTheme()->imagePath("widgets/keren");
    mKSvg.load(svgFile);

    mMaxCountDownSecond = 5;
    
    mCountDownTimer = new QTimer(this);
    connect(mCountDownTimer, SIGNAL(timeout()), this, SLOT(slotCountDownTimer()));
}

ControlWidget::~ControlWidget()
{
}

void ControlWidget::setInfoImage(const QImage &image)
{
    
}

void ControlWidget::startCountDown(int counter)
{
    mMaxCountDownSecond = counter;
    mCountDownSecond = mMaxCountDownSecond;
    mCountDownTimer->start(1000);
}

void ControlWidget::slotCountDownTimer()
{
    kDebug() << "Count Down " << mCountDownSecond;
    mCountDownTimer->stop();

    if (mCountDownSecond == 0) {
        emit countDownFinished();
    } else {
        //composeImageNumber(mCountDownSecond);

        mCountDownSecond--;
        mCountDownTimer->start(1000);
    }
}

void ControlWidget::composeImageNumber(int number)
{
    QStyleOptionGraphicsItem option;
    QPixmap tempPixmap(QSize(48, 50));
    tempPixmap.fill(Qt::transparent);
    QPainter painter(&tempPixmap);

    painter.setRenderHint(QPainter::SmoothPixmapTransform);
    painter.setRenderHint(QPainter::Antialiasing);
    mInfoImage = tempPixmap.toImage();
    this->update(mCircleButtonRect);
}

void ControlWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
    Q_UNUSED(option)
    Q_UNUSED(widget)
    QRectF contentRect = this->rect();

    
    painter->setRenderHint(QPainter::SmoothPixmapTransform);
    painter->setRenderHint(QPainter::Antialiasing);
    
    mKSvg.render(painter,"info_circle",contentRect);
}

void ControlWidget::mousePressEvent(QGraphicsSceneMouseEvent * event)
{
        qreal zScale = mSvg->elementRect("keren_control").width()/this->contentsRect().width();
        qreal xScale = this->contentsRect().width() / mSvg->elementRect("keren_control").width();
        qreal yScale = xScale;
        kDebug() << "keren_control=" << mSvg->elementRect("keren_control");
        kDebug() << "contentRect=" << this->contentsRect();
        kDebug() << "Scale=" << xScale << " Scale2=" << zScale ;
        
        QRectF infoRect = mSvg->elementRect("infocircle");
        QRectF picControlRect = mSvg->elementRect("picture_control");
        QRectF vidControlRect = mSvg->elementRect("video_control");
        
        infoRect.setRect(infoRect.x() * xScale, infoRect.y() * yScale, infoRect.width() * xScale, infoRect.height() * yScale);
        picControlRect.setRect(picControlRect.x() * xScale, picControlRect.y() * yScale, picControlRect.width() * xScale, picControlRect.height() * yScale);
        vidControlRect.setRect(vidControlRect.x() * xScale, vidControlRect.y() * yScale, vidControlRect.width() * xScale, vidControlRect.height() * yScale);
        
        
    //check which button is pressed
    if (event->button() == Qt::LeftButton) {
        QPointF pos = event->pos();
        kDebug() << "Pos=" << event->pos();
        
        if (picControlRect.contains(pos)) {
            kDebug() << "Click picture";
            emit pictureButtonClicked();
        }
        else if (vidControlRect.contains(pos)) {
            kDebug() << "Click video";
        }
        else if (infoRect.contains(pos)) {
            kDebug() << "Click info";
        }
        /*if (mPictureButtonRect.contains(pos.x(), pos.y())) {
            emit pictureButtonClicked();
        } else if (mVideoButtonRect.contains(pos.x(), pos.y())) {
            emit videoButtonClicked();
        }*/
    }
}
#include "controlwidget.moc"