/***************************************************************************
 *   Copyright © 2009 Reza Fatahilah Shah <rshah0385@kireihana.com>        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/
#ifndef CONTROL_WIDGET_H
#define CONTROL_WIDGET_H

#include <QRect>
#include <QImage>
#include <QGraphicsWidget>
#include <KSvgRenderer>

class QTimer;
class QGraphicsSceneMouseEvent;

namespace Plasma
{
    class Svg;
}

class ControlWidget : public QGraphicsWidget
{
    Q_OBJECT
public:
    ControlWidget(QGraphicsItem *parent = 0, Qt::WindowFlags wFlags = 0);
    ~ControlWidget();

    void startCountDown(int counter);
    void setInfoImage(const QImage &image);
    void paint(QPainter *painter, const QStyleOptionGraphicsItem * option, QWidget* widget = 0);
private Q_SLOTS:
    void slotCountDownTimer();

signals:
    void pictureButtonClicked();
    void videoButtonClicked();
    void countDownFinished();

protected:
    void mousePressEvent(QGraphicsSceneMouseEvent * event);
private:
    void composeImageNumber(int number);
private:
    KSvgRenderer mKSvg;
    int mMaxCountDownSecond;
    int mCountDownSecond;
    QImage mImage;
    QImage mInfoImage;
    QRect mPictureButtonRect;
    QRect mVideoButtonRect;
    QRect mCircleButtonRect;
    Plasma::Svg *mSvg;
    QTimer *mCountDownTimer;
};

#endif