/*
 *   Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
 *   Copyright © 2009 Reza Fatahilah Shah <rshah0385@kireihana.com>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public License
 *   along with this library; see the file COPYING.LIB.  If not, write to
 *   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *   Boston, MA 02110-1301, USA.
 */
/*
    This code is slim version of phonon one
*/

#ifndef VIDEO_WIDGET_H
#define VIDEO_WIDGET_H
#include <phonon/videowidget.h>
#include "abstractrenderer.h"

#include <gst/gst.h>

class VideoWidget : public QWidget
{
    Q_OBJECT
public:
    VideoWidget(QWidget *parent = 0,QString renderType="software");
    ~VideoWidget();

    void setupVideoBin();
    void paintEvent(QPaintEvent *event);
    void setVisible(bool);

    Phonon::VideoWidget::AspectRatio aspectRatio() const;
    void setAspectRatio(Phonon::VideoWidget::AspectRatio aspectRatio);
    Phonon::VideoWidget::ScaleMode scaleMode() const;
    void setScaleMode(Phonon::VideoWidget::ScaleMode);
    void setMovieSize(const QSize &size);
    QSize sizeHint() const;
    QRect scaleToAspect(QRect srcRect, int w, int h) const;
    QRect calculateDrawFrameRect() const;

    GstElement *videoBin()
    {
        Q_ASSERT(m_videoBin);
        return m_videoBin;
    }

    QSize movieSize() const {
        return m_movieSize;
    }

    bool event(QEvent *);

    QWidget *widget() {
        return this;
    }

protected:
    GstElement *m_videoBin;
    QSize m_movieSize;
    AbstractRenderer *m_renderer;

private:
    AbstractRenderer *createVideoRenderer();

private:
    Phonon::VideoWidget::AspectRatio m_aspectRatio;
    Phonon::VideoWidget::ScaleMode m_scaleMode;

    GstElement *m_colorspace;
    GstElement *m_videoplug;
    QString mRendererType;
};
#endif