/***************************************************************************
 *   Copyright © 2009 Reza Fatahilah Shah <rshah0385@kireihana.com>        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/
#include "videoeffectbin.h"
#include <KDebug>
#include "libqtgstreamer/qgstpad.h"
#include "libqtgstreamer/qgstghostpad.h"
#include "libqtgstreamer/qgstelementfactory.h"
#include <gst/gst.h>
using namespace QtGstreamer;

struct VideoEffectBin::Private
{
    QGstElementPtr capsFilter;
    QGstElementPtr tee;
};

/*bool VideoEffectBin::bufferProbe(GstPad *pad,GstBuffer *buffer,VideoEffectBin *videoBin)
{
    //kDebug() << "Buffer in ts=" << GST_BUFFER_TIMESTAMP(buffer);
    if (videoBin->adjustTimeStamp == true) {
        kDebug() << "Buffer in ts=" << GST_BUFFER_TIMESTAMP(buffer);
        videoBin->adjustTimeStamp = false;
        videoBin->deltaTimeStamp = GST_BUFFER_TIMESTAMP(buffer);
        GstEvent * newSegmentEvent = gst_event_new_new_segment(FALSE,1,GST_FORMAT_TIME,0, -1, 0);
        QGstPadPtr sinkPad;
        sinkPad = videoBin->d->tee->getStaticPad("sink");
        gst_pad_send_event(GST_PAD(sinkPad->peekNativeObject()),newSegmentEvent);
        GST_BUFFER_FLAG_SET(buffer,GST_BUFFER_FLAG_DISCONT);
        //kDebug() << "Buffer out ts=" << GST_BUFFER_TIMESTAMP(buffer);
    }
    GST_BUFFER_TIMESTAMP(buffer) = GST_BUFFER_TIMESTAMP(buffer) - videoBin->deltaTimeStamp;
    //kDebug() << "Buffer out ts=" << GST_BUFFER_TIMESTAMP(buffer);
    return true;
}
*/

void VideoEffectBin::newTimeStamp()
{
    adjustTimeStamp = true;
}

VideoEffectBinPtr VideoEffectBin::createVideoEffectBin()
{
    QGstElementPtr tee = QGstElementFactory::make("tee");
    QGstElementPtr identity = QGstElementFactory::make("identity");
    
    if ( !identity || !tee ) {
        return VideoEffectBinPtr();
    }

    VideoEffectBin *videoEffectBin = new VideoEffectBin();

    videoEffectBin->add(identity);    
    videoEffectBin->add(tee);
    videoEffectBin->d->tee = tee;

    QGstElement::link(identity,tee);

    QGstPadPtr pad = tee->getRequestPad("src%d");
    QGstGhostPadPtr ghostPad = QGstGhostPad::newGhostPad("src0",pad);
    videoEffectBin->addPad(ghostPad);

    pad = tee->getRequestPad("src%d");
    ghostPad = QGstGhostPad::newGhostPad("src1",pad);
    videoEffectBin->addPad(ghostPad);

    pad = identity->getStaticPad("sink");
    ghostPad = QGstGhostPad::newGhostPad("sink",pad);
    videoEffectBin->addPad(ghostPad);

    /*pad = videoEffectBin->getStaticPad("src1");
    gst_pad_add_buffer_probe(GST_PAD(pad->peekNativeObject()),G_CALLBACK(&bufferProbe),videoEffectBin);*/

    return VideoEffectBinPtr(videoEffectBin);
}

VideoEffectBin::VideoEffectBin()
    : QGstBin("video_effect_bin"), d(new Private),adjustTimeStamp(false)
{
}

VideoEffectBin::~VideoEffectBin()
{
    delete d;
}

//multiple effect codes
/*else {
        mFrameSize=QSize(480,360);
        GstElement *videoScale,*videoScaleCapsFilter,*afterScaleTee,*beforeScaleQueue;
        afterScaleTee = gst_element_factory_make("tee","after_scale_tee");
        videoScale = gst_element_factory_make("videoscale","video_scale");
        beforeScaleQueue = gst_element_factory_make("queue","video_scale_queue");
        videoScaleCapsFilter = gst_element_factory_make("capsfilter","scale_caps_filter");

        GstCaps *scaleCaps = gst_caps_new_simple("video/x-raw-yuv", 
                        "width", G_TYPE_INT, 160,"height", G_TYPE_INT, 120,NULL);

        g_object_set(G_OBJECT(videoScaleCapsFilter), "caps", scaleCaps,NULL);
        gst_caps_unref(scaleCaps);

        gst_bin_add_many(GST_BIN(mVideoEffectBin),beforeScaleQueue,videoScale,videoScaleCapsFilter,afterScaleTee,NULL);
        gst_element_link_many(mEffectFilter,beforeScaleQueue,videoScale,videoScaleCapsFilter,afterScaleTee,NULL);

        char *effectName[] = { "vertigotv","shagadelictv","warptv","dicetv","identity"};
        int totalEffect = 5;

        GstElement *effectFilter;
        GstElement *videoEffectMixer;
        GstPad *src,*pad;
        int maxXpos=320;
        int xpos=0;
        int ypos=0;
        int width=160;
        int height=120;

        videoEffectMixer = gst_element_factory_make("videomixer","video_effect_mixer");        
        gst_bin_add_many(GST_BIN(mVideoEffectBin),videoEffectMixer,NULL);    

        GstElement *videoSrc = gst_element_factory_make("videotestsrc",NULL);
        GstElement *testCapsFilter = gst_element_factory_make("capsfilter",NULL);
        GstCaps *testCaps = gst_caps_new_simple("video/x-raw-yuv", 
                        "width", G_TYPE_INT, mFrameSize.width(),"height", G_TYPE_INT, mFrameSize.height(),"framerate",GST_TYPE_FRACTION,5,1,NULL);
        g_object_set(G_OBJECT(testCapsFilter), "caps", testCaps,NULL);
        gst_caps_unref(testCaps);

        g_object_set (videoSrc, "pattern", 2,NULL);

        gst_bin_add_many(GST_BIN(mVideoEffectBin),videoSrc,testCapsFilter,NULL);         
        gst_element_link_many(videoSrc,testCapsFilter,NULL);

        src = gst_element_get_static_pad(testCapsFilter,"src");
        pad = gst_element_get_request_pad(videoEffectMixer,"sink_%d");
        gst_pad_link(src,pad);

        for (int ii=0;ii<totalEffect;ii++) {
            effectFilter = getEffectBin(effectName[ii]);
            gst_bin_add_many(GST_BIN(mVideoEffectBin),effectFilter,NULL);
            gst_element_link_many(afterScaleTee,effectFilter,NULL);

            src = gst_element_get_static_pad(effectFilter,"src");
            pad = gst_element_get_request_pad(videoEffectMixer,"sink_%d");
            g_object_set (pad, "zorder",1,"xpos", xpos, "ypos", ypos,NULL);
            gst_pad_link(src,pad);
            xpos += width;
            if (xpos>maxXpos) {
                xpos = 0;
                ypos += height;
            }
        }

        gst_element_link_many(videoEffectMixer,effectSourceTee,NULL);
    }        
    */
/*GstElement * VideoGst::getEffectBin(const char* effectName)
{
    GstElement *effectBin,*queue,*effect,*cspBeforeEffect,*cspAfterEffect;
    GstPad *pad;

    effectBin=gst_bin_new(NULL);

    queue = gst_element_factory_make("queue",NULL);
    effect = gst_element_factory_make(effectName,NULL);
    cspBeforeEffect = gst_element_factory_make("ffmpegcolorspace",NULL);
    cspAfterEffect = gst_element_factory_make("ffmpegcolorspace",NULL);

    gst_bin_add_many(GST_BIN(effectBin),queue,cspBeforeEffect,effect,cspAfterEffect,NULL);
    gst_element_link_many(queue,cspBeforeEffect,effect,cspAfterEffect,NULL);
    
    pad = gst_element_get_static_pad(queue,"sink");
    gst_element_add_pad (effectBin, gst_ghost_pad_new ("sink", pad));
    gst_object_unref (GST_OBJECT (pad));

    pad = gst_element_get_static_pad(cspAfterEffect,"src");
    gst_element_add_pad (effectBin, gst_ghost_pad_new ("src", pad));
    gst_object_unref (GST_OBJECT (pad));

    return effectBin;
}*/
#include "videoeffectbin.moc"