/***************************************************************************
 *   Copyright © 2009 Reza Fatahilah Shah <rshah0385@kireihana.com>        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/
#ifndef VIDEO_INPUT_DEVICE_H
#define VIDEO_INPUT_DEVICE_H
#include <QList>
#include <QSize>
#include "libqtgstreamer/qgstelement.h"

class VideoInputDevice;
typedef QSharedPointer<VideoInputDevice> VideoInputDevicePtr;

class VideoInputDevicePrivate;
class VideoInputDevice : public QtGstreamer::QGstElement
{
    Q_OBJECT
    Q_DISABLE_COPY(VideoInputDevice)
public:
    friend class VideoInputDevicePrivate;
    
    static VideoInputDevicePtr newDevice(const QString &driver, const QString &devicePath);
    virtual ~VideoInputDevice();

    void queryDeviceData();
    QList<QSize> getSupportedResolution() const;
    QList<QSize> getSupportedFrameRate(const QSize &resolution) const;

    bool hasColorBalance();
    int brightness();
    void setBrightness(int value);
    void setContrast(int value);
    int contrast();
    void setSaturation(int value);
    int saturation();
    void setHue(int value);
    int hue();

protected:
    VideoInputDevice(GstElement *element);
    QList<QSize> queryFramerates(const QtGstreamer::QGstStructure &capsStructure);

private:
    VideoInputDevicePrivate *const d;
};

#endif