/*
 * This file is part of the KDE project
 * Copyright (C) 2009 Eva Brucherseifer <eva.brucherseifer@basyskom.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "killswitchitem.h"

#include <QMap>


KillSwitchItem::KillSwitchItem(KillSwitchDevice * device, QGraphicsItem* parent)
: QGraphicsWidget(parent), m_rfDevice(device)
{
    //setAcceptHoverEvents(true);

    m_layout = new QGraphicsLinearLayout(Qt::Horizontal, this);
    m_layout->setSpacing(0);
    //m_layout->setItemSpacing(8);
//    m_layout->setVerticalSpacing(0);
//    m_layout->setColumnSpacing(0, 8);
//    m_layout->setColumnSpacing(1, 4);
//    m_layout->setColumnSpacing(2, 6);
    m_layout->setPreferredWidth(100);
//    m_layout->setColumnFixedWidth(0, 48);
//    m_layout->setColumnMinimumWidth(1, 144);
//    m_layout->setColumnFixedWidth(2, 60); // FIXME: spacing?
//    m_layout->setColumnFixedWidth(3, 22); // FIXME: spacing?

    m_icon = new Plasma::IconWidget(this);
    m_icon->setMinimumHeight(48);
    m_icon->setMaximumHeight(48);
    //m_icon->setAcceptHoverEvents(false);
//    m_layout->addItem(m_icon, 0, 0, 1, 1);
    m_layout->addItem(m_icon);

    QString type = m_rfDevice->type();
    if (type=="wireless")
            m_icon->setIcon("network-wireless");
    else if (type == "wwan")
            m_icon->setIcon("phone");
    else if (type =="bluetooth")
            m_icon->setIcon("preferences-system-bluetooth");
    else 
            m_icon->setIcon("network-wireless");

    m_rfCheckBox = new Plasma::CheckBox(this);
//    m_rfCheckBox->setText(i18n("Enable"));
    m_rfCheckBox->setToolTip(i18nc("CheckBox to enable or disable interface (rfkill)", "Enable Device"));
//    m_layout->addItem(m_rfCheckBox, 0, 1, 1, 1, Qt::AlignLeft);
    m_layout->addItem(m_rfCheckBox);

    updateState();

    connect(m_rfCheckBox, SIGNAL(toggled(bool)),
            SLOT(killswitchToggled(bool)));
    QObject::connect(m_rfDevice, SIGNAL(propertyChanged(const QMap<QString,int>&)), SLOT(updateState()));
}

KillSwitchItem::~KillSwitchItem()
{
//    delete m_rfDevice;
}

KillSwitchDevice* KillSwitchItem::device()
{
    return m_rfDevice;
}

void KillSwitchItem::killswitchToggled(bool checked)
{
    if (!m_rfCheckBox->isEnabled())
        return;
    if (checked)
        m_rfDevice->setPower(true);
    else 
        m_rfDevice->setPower(false);
}


void KillSwitchItem::updateState()
{
    int state = m_rfDevice->state();
    switch (state)
    {
        case 0:
            m_rfCheckBox->setEnabled(true);
            m_rfCheckBox->setChecked(false);
            break;
        case 1:
            m_rfCheckBox->setEnabled(true);
            m_rfCheckBox->setChecked(true);
            break;
        case 2:
        default:
            m_rfCheckBox->setEnabled(false);
            m_rfCheckBox->setChecked(false);
            break;
    }
     
    QString text = m_rfDevice->type() + " - " + QString::number(state); 
        + " " + m_rfDevice->name() + " " + m_rfDevice->access_method();
    kDebug() << text;
}

#include "killswitchitem.moc"
