/***************************************************************************
 *   Copyright (C) 2009 by Björn Ruberg <bjoern@ruberg-wegener.de          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include <QGraphicsGridLayout>
#include "contactlistitem.h"
#include "contact.h"
#include "KopeteExtender.h"
#include <KIcon>

ContactListItem::ContactListItem(QGraphicsWidget *parent, Contact *contact) : Plasma::Frame(parent){

    m_contact = contact;

    m_layout = new QGraphicsGridLayout ( this );
    m_layout->setSpacing(1);

    avatar = new Plasma::IconWidget( this );
    avatar->setIcon ( *contact->getAvatar()  );
    avatar->setMinimumWidth(40);
    m_layout->addItem(avatar,0,0,2,1);

    nick = new Plasma::Label(this);
    nick->setText( contact -> getDisplayName() );
    nick->setSizePolicy ( QSizePolicy::Expanding, QSizePolicy::Expanding, QSizePolicy::DefaultType );
    m_layout->addItem (nick,0,1);

    message = new Plasma::Label(this);
    message->setText( contact -> getStatusMessage());
    message->setSizePolicy ( QSizePolicy::Expanding, QSizePolicy::Expanding, QSizePolicy::DefaultType );
    m_layout->addItem (message,1,1);

    setPreferredSize(30, 150);

    connect(avatar, SIGNAL(clicked()), this , SLOT(activated()));
}

ContactListItem::~ContactListItem(){
    m_layout->removeAt(2);
    m_layout->removeAt(1);
    m_layout->removeAt(0);
}

void ContactListItem::setIcon(KIcon* icon){
    avatar->setIcon (*icon);
}

void ContactListItem::setNick(QString text){
    nick->setText(text);
}

void ContactListItem::setVisible(bool visible){
    Plasma::Frame::setVisible(visible);
    avatar->setVisible(visible);
    nick->setVisible(visible);
    message->setVisible(visible);
}

void ContactListItem::activated(){
    ((KopeteExtender*)(parentItem()->parentItem()))->openChat(m_contact);
}
