/***********************************************************************************
* STasks
* Copyright (C) 2009 Marcin Baszczewski <marcin.baszczewski@gmail.com>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
***********************************************************************************/

#include "windowpreview.h"

#include <QPainter>

//X11
#include <QX11Info>
#include <X11/Xlib.h>
#include <fixx11h.h>


#include <KWindowSystem>
#include <KDebug>

//OTHER
#include <taskmanager/taskmanager.h>
#include <taskmanager/abstractgroupableitem.h>
#include <taskmanager/groupmanager.h>
#include <taskmanager/taskitem.h>
#include <taskmanager/startup.h>
#include <taskmanager/taskactions.h>

WindowPreview::WindowPreview(QSize size, WId window, QWidget *parent) : QWidget(parent)
{
    m_window = window;

    m_opacity = 0.75;

    m_background = new Plasma::FrameSvg(this);
    m_background->setImagePath("widgets/frame");
    m_background->setElementPrefix("raised");

    qreal left, top, right, bottom;

    m_background->getMargins(left, top, right, bottom);

    QSize fSize = QSize((size.width() + left + right), (size.height() + top + bottom));

    m_background->resizeFrame(fSize);
    setFixedSize(fSize);

    updateTheme();

    connect(m_background, SIGNAL(repaintNeeded()), this, SLOT(updateTheme()));
}

WindowPreview::~WindowPreview()
{
    delete m_background;
}

QRect WindowPreview::thumbnailRect()
{
    qreal left, top, right, bottom;

    m_background->getMargins(left, top, right, bottom);

    return geometry().adjusted(left, top, -right, -bottom);
}

void WindowPreview::paintEvent(QPaintEvent *event)
{
    Q_UNUSED(event)

    QPainter painter(this);
    painter.setOpacity(m_opacity);

    m_background->paintFrame(&painter);
}

void WindowPreview::enterEvent(QEvent *event)
{
    Q_UNUSED(event)

    m_opacity = 1;
    
    highlightWindow(parentWidget());
  
    update();
}

void WindowPreview::leaveEvent(QEvent *event)
{
    Q_UNUSED(event)

    m_opacity = 0.75;

    unHighlightWindow(parentWidget());
    update();
}

void WindowPreview::mousePressEvent(QMouseEvent *event)
{
    KWindowSystem::forceActiveWindow(m_window);
    
    m_opacity = 1;
    event->ignore();
}

void WindowPreview::updateTheme()
{
    m_background->clearCache();

    update();
}

void WindowPreview::highlightWindow(QWidget *caller)
{
    Display *dpy = QX11Info::display();
    Atom atom = XInternAtom(dpy, "_KDE_WINDOW_HIGHLIGHT", False);
    long data[] = { m_window };
    XChangeProperty(dpy, caller->winId(), atom, atom, 32, PropModeReplace,
        reinterpret_cast<unsigned char *>(data), sizeof(data) / sizeof(data[ 0 ]));
}

void WindowPreview::unHighlightWindow(QWidget *caller)
{
   Display *dpy = QX11Info::display();
    Atom atom = XInternAtom(dpy, "_KDE_WINDOW_HIGHLIGHT", False);
    long data[] = { NULL };
    XChangeProperty(dpy, caller->winId(), atom, atom, 32, PropModeReplace,
        reinterpret_cast<unsigned char *>(data), sizeof(data) / sizeof(data[ 0 ]));
}
