#include "trenditem.h"
#include <QPainter>
#include <QFontMetrics>
#include <QtDebug>

TrendItem::TrendItem(QGraphicsItem *parent)
    : SvgLayoutItem(parent)
{
    m_minSize = QSizeF(100.0, 25.0);
    setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::MinimumExpanding));
    setImagePath("widgets/stock_bar_negative");
    m_dblPercentChange = 0.0;
    m_dblChange = 0.0;
}

TrendItem::~TrendItem()
{
}

void TrendItem::setChange(qreal dblChange)
{
    m_dblChange = dblChange;
    if (m_dblChange < 0.0)
    {
        setImagePath("widgets/stock_bar_negative");   
    }
    else if (m_dblChange > 0.0)
    {
        setImagePath("widgets/stock_bar_positive");
    }
    else
    {
        setImagePath("widgets/stock_bar_neutral");
    }
    update();
}

qreal TrendItem::change()
{
    return m_dblChange;
}

void TrendItem::setPercentChange(qreal dblPercentChange)
{
    m_dblPercentChange = dblPercentChange;
    QString sTmp = QString("%1%").arg(m_dblPercentChange, 0, 'g', 2);
    QFont fontBig;
    fontBig.setStyleHint(QFont::SansSerif);
    fontBig.setWeight(QFont::Bold);
    fontBig.setPointSize(12);
    QFontMetrics m(fontBig);

    QRect r = m.boundingRect(sTmp);
//    m_minSize = QSizeF(r.width()+10, r.height()+10);
  //  updateGeometry();
}

qreal TrendItem::percentChange()
{
    return m_dblPercentChange;
}

void TrendItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
    QRectF rTextRect = boundingRect().adjusted(5, 5, -5, -5);
    QFont fontBig;
    fontBig.setStyleHint(QFont::SansSerif);
    fontBig.setWeight(QFont::Bold);
    fontBig.setPointSize(12);

    SvgLayoutItem::paint(painter, option, widget);
    painter->setPen(QPen(Qt::white));
    painter->setFont(fontBig);
    painter->drawText(rTextRect, Qt::AlignCenter | Qt::AlignVCenter, QString("%1%").arg(m_dblPercentChange, 0, 'g', 2));
}

QSizeF TrendItem::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
{
    switch(which)
    {
    case Qt::MinimumSize:
        return m_minSize;
    case Qt::PreferredSize:
        return m_minSize;
    default:
        return QSizeF();
    }
}

