/*
 *   Copyright (C) 2007 Mathias Soeken <msoeken@tzi.de>
 *   Copyright (C) 2008 Montel Laurent <montel@kde.org>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef KTIMETRACKERAPPLET_H
#define KTIMETRACKERAPPLET_H

#include <QGraphicsProxyWidget>
#include <plasma/applet.h>
#include <plasma/dataengine.h>
#include "ui_timetrackerConfig.h"

class QGraphicsLinearLayout;
namespace Plasma
{
    class Label;
    class IconWidget;
    class PushButton;
    class LineEdit;
}

class KTimetrackerLaunchWidget : public QGraphicsProxyWidget
{
    Q_OBJECT
public:
    KTimetrackerLaunchWidget(QGraphicsWidget *parent = 0);
    ~KTimetrackerLaunchWidget();
private slots:
    void startApplication();
private:
    QGraphicsLinearLayout *m_layout;
    Plasma::Label *m_errorLabel;
    Plasma::IconWidget *m_icon;
    Plasma::PushButton *m_launchButton;
};

class KTimetrackerAppWidget : public QGraphicsProxyWidget
{
    Q_OBJECT
public:
    KTimetrackerAppWidget(QGraphicsWidget *parent = 0);
    ~KTimetrackerAppWidget();
    void changeActiveLabelTask( const QString &taskName );
private slots:
    void startCurrentTask();
    void stopCurrentTask();
protected:
    bool currentTaskIsActive() const;
    void updateButtons();

private:
    Plasma::Label *mCurrentTask;
    QGraphicsLinearLayout *m_layout;
    Plasma::IconWidget *m_icon;

    Plasma::IconWidget *m_start;
    Plasma::IconWidget *m_stop;
};


class KTimetrackerApplet : public Plasma::Applet
{
Q_OBJECT
public:
  KTimetrackerApplet(QObject *parent, const QVariantList &args);
  ~KTimetrackerApplet();

    void init();
public Q_SLOTS:
  void dataUpdated(const QString &name, const Plasma::DataEngine::Data &data);

protected:
    void createConfigurationInterface(KConfigDialog *parent);

private Q_SLOTS:
  void configAccepted();

protected:
    void addAppWidget();
    void addLaunchTimerTrackerWidget();

private:
    Ui::timetrackerConfig mConfigWidget;
    Plasma::DataEngine *mEngine;
    QGraphicsLinearLayout *mLayout;
    KTimetrackerLaunchWidget *mLaunchWidget;
    KTimetrackerAppWidget *mAppWidget;
};

K_EXPORT_PLASMA_APPLET(timetrackerapplet, KTimetrackerApplet)

#endif
