/*
# * Copyright(c) 2008 Matthew Adams <roguestar191 at comcast.net>
# *
# * This program is free software; you can redistribute it and/or
# * modify it under the terms of the GNU General Public License as
# * published by the Free Software Foundation; either version 2 of
# * the License, or(at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program.  If not, see <http//www.gnu.org/licenses/>.
# */
#include <../includes/TipDatabase.h>
#include <fstream>
#include <cstdlib>
#include <vector>
int randomNumber(int lowest, int highest) {
	int range=(highest-lowest)+1;	
	return lowest+int(range*rand()/(RAND_MAX + 1.0));
}
namespace welcome_plasmoid {
	class aTip {
		public:
		aTip(QString t, QString c) {
			tip = t;
			category = c;
		}
		aTip(const aTip& old) {
			tip = old.tip;
			category = old.category;
		}
		QString tip;
		QString category;
	};
	class TipDatabasePrivate {
		TipDatabase *top;
		bool html;
		int currentTip;
		public:
		QString file;
		std::vector<aTip> tips;
		public:
			
			TipDatabasePrivate(TipDatabase*_top, QString &filename) : top(_top){
				load(filename);
				currentTip = randomNumber(0, tips.size() );
			}
			TipDatabasePrivate(TipDatabase*_top) : top(_top) {
				html = true;
			}
			QString tip() {
				if(html) return "";

				if (tips.size() == 0) return "Error, no tips! in tipfile: " + file; 
				QString ret = "<html>Category: ";
				ret.append(tips[currentTip].category);
				ret.append("<br>Did you know...<br><br>");
				ret.append(tips[currentTip].tip);
				return ret;
//				return "<html>Category:" + tips[currentTip].category + "<br>Did you know...<br><br> " + tips[currentTip].tip;
			}
			void prevTip() {
				currentTip--;
				if(currentTip < 0) currentTip = tips.size();
			}
			void nextTip() {
				currentTip++;
				if(currentTip >= tips.size() ) currentTip = 0;
			}
			bool isHtml() { return html; }
			void load(QString &filename) {
				filename.replace("file:", "");
				file = filename;
				if(filename.contains(".html")) html = true;
				else html = false;
				if(html) return;
				if(filename == "") return;
				tips.clear();
				std::ifstream infile(filename.toLocal8Bit().data());
				QString data;
				{
					std::string buf;
					while( getline(infile, buf) ) data.append(buf.c_str());
				}
				infile.close();
				QStringList list = data.split("</tip>");
				for(QStringList::Iterator iter = list.begin(), end = list.end(); iter != end; iter++) {
					if( (*iter).length() > 6) {
						QString category = "Unknown";
						int cat = (*iter).indexOf("category=");
						if(cat != -1 ){
							category = (*iter).mid(cat+10, (*iter).length()-cat+10);
							int findclose = (*iter).indexOf("\"", cat+10);
							(*iter) = (*iter).right( (*(iter)).length()-(*iter).indexOf(">", findclose)-1 ); 

							(*iter).replace("</tip>", "");
							(*iter).replace("<html>", "");
							findclose = category.indexOf("\"", 0);
							category = category.left(findclose);
						} else {
							int end = (*iter).indexOf(">", 0);
							(*iter) = (*iter).right(end+1);
						}

						aTip onetip( (*iter), category);
						tips.push_back(onetip);
					}
				}
				currentTip = randomNumber(0, tips.size() );
			}
	};
	TipDatabase::TipDatabase()  { 
		priv = new TipDatabasePrivate(this);
	}
	TipDatabase::TipDatabase(QString&file)  {
		priv = new TipDatabasePrivate(this, file);
	}
	TipDatabase::TipDatabase(const TipDatabase& old ) {
		priv = new TipDatabasePrivate(this, ((TipDatabasePrivate*)old.priv)->file);
	}
	TipDatabase::~TipDatabase() {
		delete ((TipDatabasePrivate*)priv);
	}
	QString TipDatabase::tip() {
		return ((TipDatabasePrivate*)priv)->tip();
	}
	bool TipDatabase::isHtml() {
		return ((TipDatabasePrivate*)priv)->isHtml();
	}
	void TipDatabase::prevTip() {
		((TipDatabasePrivate*)priv)->prevTip();
	}
	void TipDatabase::nextTip() {
		((TipDatabasePrivate*)priv)->nextTip();
	}
	void TipDatabase::load(QString&file) {
		((TipDatabasePrivate*)priv)->load(file);
	}
}

