/*
*   Copyright 2007 by Alex Merry <alex.merry@kdemail.net>
*   Copyright 2008 by Alexis Ménard <darktears31@gmail.com>
*
*
*   This program is free software; you can redistribute it and/or modify
*   it under the terms of the GNU Library General Public License version 2,
*   or (at your option) any later version.
*
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details
*
*   You should have received a copy of the GNU Library General Public
*   License along with this program; if not, write to the
*   Free Software Foundation, Inc.,
*   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef WELCOME_PLASMA_PANEL_H
#define WELCOME_PLASMA_PANEL_H

#include <Plasma/Containment>

class QComboBox;
class QAction;
class KDialog;
class KIntNumInput;
namespace welcome_plasmoid
{
    class Spacer;
}
namespace Plasma
{
    class FrameSvg;
}
namespace welcome_plasmoid 
{
class Panel : public Plasma::Containment
{
    Q_OBJECT
public:
    Panel(QObject *parent, const QVariantList &args);
    Panel(QGraphicsWidget*, QString, uint&);
    ~Panel();
    void init();
    QList<QAction*> contextualActions();

    virtual void constraintsEvent(Plasma::Constraints constraints);

    void paintInterface(QPainter *painter,
                        const QStyleOptionGraphicsItem *option,
                        const QRect &contentsRect);
    void paintBackground(QPainter *painter, const QRect &contentsRect);
    void showDropZone(const QPoint pos);
    bool isContainment() const; 
protected:
    void saveState(KConfigGroup &config) const;

protected slots:
    void themeUpdated();
    void backgroundChanged();
public slots:    
    void layoutApplet(Plasma::Applet* applet, const QPointF &pos);
    Plasma::Applet *addApplet(const QString &name, const QVariantList &args = QVariantList(),
                           const QRectF &geometry = QRectF(-1, -1, -1, -1), uint appletid = 0, bool dontInit = false);
 
    void addApplet(Applet *applet, const QPointF &pos = QPointF(-1, -1), bool dontInit = true);
//    Plasma::Applet *addApplet(const QString &name, const QVariantList &args, const QRectF &appletGeometry, uint id, bool delayInit)
//    void addApplet(Applet *applet, const QPointF &pos, bool delayInit);
protected slots:
    void appletRemoved(Plasma::Applet* applet);
    void updateSize();
    void addPanel();
private:
    /**
     * update the formfactor based on the location
     */
    void setFormFactorFromLocation(Plasma::Location loc);

    /**
     * recalculate which borders to show
     */
    void updateBorders(const QRect &geom);

    Plasma::FrameSvg *m_background;
    QAction* m_configureAction;
    QAction* m_addPanelAction;

    //cached values
    QSize m_currentSize;
    QRect m_lastViewGeom;
    bool m_maskDirty;
    int m_spacerIndex;
    Spacer *m_spacer;

    friend class Spacer;
};
}

#endif // WELCOME_PLASMA_PANEL_H

