/*
    Copyright (C) 2010 Kevin Krammer <kevin.krammer@gmx.at>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "agentinstancemethodjob.h"

#include <Akonadi/AgentInstance>
#include <Akonadi/AgentManager>

#include <KDebug>

#include <QWidget>

using namespace Akonadi;

class AgentInstanceMethodJob::Private
{
  AgentInstanceMethodJob *const q;

  public:
    Private( AgentInstanceMethodJob *parent )
      : q( parent )
    {
    }

    void callAgentInstanceMethod();
};

void AgentInstanceMethodJob::Private::callAgentInstanceMethod()
{
  AgentInstance agent = AgentManager::self()->instance( q->destination() );
  if ( !agent.isValid() ) {
    kDebug() << "Given agent instance is not or no longer valid";
    q->setResult( false );
    return;
  }

  if ( q->operationName() == QLatin1String( "synchronizeCollectionTree" ) ) {
    agent.synchronizeCollectionTree();
  } else if ( q->operationName() == QLatin1String( "synchronizeAll" ) ) {
    agent.synchronize();
  } else if ( q->operationName() == QLatin1String( "configure" ) ) {
    // see if our parent is a widget. if so, use it as parent for configure
    QWidget *widget = qobject_cast<QWidget*>( q->parent() );
    agent.configure( widget );
  } else if ( q->operationName() == QLatin1String( "restart" ) ) {
    agent.restart();
  } else {
    q->setResult( false );
    return;
  }

  q->setResult( true );
}

AgentInstanceMethodJob::AgentInstanceMethodJob( const QString &identifier, const QString &operation, const QMap<QString, QVariant> &parameters, QObject *parent )
  : Plasma::ServiceJob( identifier, operation, parameters, parent ),
    d( new Private( this ) )
{
}

AgentInstanceMethodJob::~AgentInstanceMethodJob()
{
  delete d;
}

void AgentInstanceMethodJob::start()
{
  QMetaObject::invokeMethod( this, "callAgentInstanceMethod", Qt::QueuedConnection );
}

QStringList AgentInstanceMethodJob::methods()
{
  return QStringList() << QLatin1String( "synchronizeCollectionTree" )
                       << QLatin1String( "synchronizeAll" )
                       << QLatin1String( "configure" )
                       << QLatin1String( "restart" );
}

#include "agentinstancemethodjob.moc"

// kate: space-indent on; indent-width 2; replace-tabs on;
