/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Copyright (C) 2008 by Dennis Nienhüser <earthwings@gentoo.org>        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef CONTACT_SOURCE_H
#define CONTACT_SOURCE_H

#include <QtCore/QObject>

#include "contact.h"

/**
 * Manages a set of contacts from a common place. Fetches their data and
 * announces contact additions and removals.
 */
class ContactSource: public QObject
{
Q_OBJECT

public:
	/** Initialize, probably gathering an initial contact list */
	virtual void initialize()	;

Q_SIGNALS	:

	/**
	 * A new contact is available
	 * @param contact New contact
	 */
	void newContact(Contact::Ptr contact);

	/**
	 * A previously available contact is no longer available
	 * @param contact Removed contact
	 */
	void contactRemoved(Contact::Ptr contact);
	
	/**
	 * Contact data has changed
     * @param contact Contact with changed properties
	 */
	void contactChanged(Contact::Ptr);
};

#endif // CONTACT_SOURCE_H
