/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Copyright (C) 2008 by Dennis Nienhüser <earthwings@gentoo.org>        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef KOPETE_SOURCE_H
#define KOPETE_SOURCE_H

#include "contactsource.h"

#include <QtCore/QObject>
#include <QtDBus/QDBusConnectionInterface>
#include <QtCore/QMap>
#include <QtCore/QString>

class KopeteContact;

static const QString KOPETE_DBUS_SERVICE = "org.kde.kopete";
static const QString KOPETE_DBUS_PATH = "/Kopete";

/**
 * Manages Kopete contacts. Fetches their data and announces contact additions
 * and removals.
 */
class KopeteSource: public ContactSource
{
Q_OBJECT

public:
	/**
	 * Reimplemented from ContactSource. Get the contact list from dbus.
	 */
	virtual void initialize();

private	Q_SLOTS:
	/** Watch out for new or removed contacts */
	void update();
	
	/** A kopetecontact has changed */
	void slotContactChanged(KopeteContact *contact);

private:
	/** Current contact list */
	QMap<QString,Contact::Ptr> m_contacts;
};

#endif // KOPETE_SOURCE_H
