/*
 *   Copyright (C) 2007 Ivan Cukic <ivan.cukic+kde@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License version 2 (or, at
 *   your option, any later version) as published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef KIOBROWSERENGINE_H
#define KIOBROWSERENGINE_H

#include "plasma/dataengine.h"
#include <QMap>
#include <QMultiMap>

#include <kio/job.h>

class KFileItem;

/**
 * This engine retreives list of files on a specific kio location
 */
class KioBrowserEngine : public Plasma::DataEngine {
    Q_OBJECT

public:
    KioBrowserEngine( QObject* parent, const QVariantList& args );
    ~KioBrowserEngine();

protected:
    bool sourceRequestEvent(const QString &path);
    void init();
    bool updateSource (const QString &source);

private Q_SLOTS:
    void entries (KIO::Job *job, const KIO::UDSEntryList &list);
    void result(KJob *job);
    void removeSource(const QString &source);
    
private:
    class SourceData {
    public:
        SourceData() : job(NULL) {};
        
        QList < KFileItem * > entries;
        QString source;
        KIO::Job * job;
        
    };
    
    KIO::ListJob * createListJob(const QString & url);
    
    QMap < QString, SourceData > m_sources;
    QMap < KIO::Job *, SourceData * > m_jobs;
};

K_EXPORT_PLASMA_DATAENGINE(kiobrowser, KioBrowserEngine)

#endif
