/*
 *   Copyright (C) 2009 Matteo Agostinelli <agostinelli@gmail.com>
 *   Copyright (C) 2007 Barış Metin <baris@pardus.org.tr>
 *   Copyright (C) 2006 David Faure <faure@kde.org>
 *   Copyright (C) 2007 Richard Moore <rich@kde.org>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "qalculate_runner.h"
#include "qalculate_engine.h"

#include <QTimer>

#include <KIcon>
#include <stdexcept>

QalculateRunner::QalculateRunner( QObject* parent, const QVariantList &args )
    : Plasma::AbstractRunner(parent, args), m_engine(0)
{
    Q_UNUSED(args)

    m_engine = new QalculateEngine;	
    setSpeed(SlowSpeed);

    setObjectName("Qalculate");
    setIgnoredTypes(Plasma::RunnerContext::Directory | Plasma::RunnerContext::File |
                         Plasma::RunnerContext::NetworkLocation | Plasma::RunnerContext::Executable |
                         Plasma::RunnerContext::ShellCommand);

    QString description = i18n("Calculates the value of :q: when :q: is made up of numbers and "
                               "mathematical symbols such as +, -, /, * and ^ using the Qalculate! library.");
    addSyntax(Plasma::RunnerSyntax("=:q:", description));
    addSyntax(Plasma::RunnerSyntax(":q:=", description));
}

QalculateRunner::~QalculateRunner()
{
    delete m_engine;
}

void QalculateRunner::match(Plasma::RunnerContext &context)
{    
    const QString term = context.query();
    QString cmd = term;
    
    cmd = cmd.trimmed().replace(" ", "");
    
    if (cmd.length() < 4) {
        return;
    }
    
    //make sure there is an equals sign. if not
    //we shouldn't be computing anything!
    if (!cmd.contains("=", Qt::CaseSensitive)) {
        return;
    }
    
    cmd.remove("=", Qt::CaseSensitive);

    if (cmd.isEmpty()) {
        return;
    }

    QString result=calculate(cmd);

    if (!result.isEmpty() && result != cmd) {
        Plasma::QueryMatch match(this);
        match.setType(Plasma::QueryMatch::InformationalMatch);
        match.setIcon(KIcon("qalculate-applet"));
        match.setText(result);
        match.setData("= " + result);
        match.setId(QString());
        context.addMatch(term, match);
    }
}

QString QalculateRunner::calculate(const QString& term)
{
    QString result = "";

    try {
        result = m_engine->evaluate(term);
    } catch(std::exception& e) {
        
    }

    return result;
}

#include "qalculate_runner.moc"
