/*
 *   Copyright 2009 Beat Wolf <asraniel@fryx.ch>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License as
 *   published by the Free Software Foundation; either version 2, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <KApplication>
#include <KCmdLineArgs>
#include <KAboutData>
#include <KUrl>
#include <KDebug>

#include "converter.h"

int main (int argc, char *argv[])
{
  KAboutData aboutData(
                         // The program name used internally.
                         "plasma2jolie",
                         // The message catalog name
                         // If null, program name is used instead.
                         0,
                         // A displayable program name string.
                         ki18n("Plasma2Jolie"),
                         // The program version string.
                         "1.0",
                         // Short description of what the app does.
                         ki18n("Transforms a plasma service to a Jolie interface"),
                         // The license this code is released under
                         KAboutData::License_GPL,
                         // Copyright Statement
                         ki18n("(c) 2009"),
                         // Optional text shown in the About box.
                         // Can contain any information desired.
                         ki18n("Some text..."),
                         // The program homepage string.
                         "",
                         // The bug report email address
                         "asraniel@fryx.ch");

  KCmdLineArgs::init( argc, argv, &aboutData ); 

  KCmdLineOptions options;
  options.add("+[file-in]", ki18n("Plasma .operations input file"));
  options.add("+[file-out]", ki18n("Jolie .iol output file"));
  KCmdLineArgs::addCmdLineOptions(options);

  KApplication app;

  KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
  if(args->count() == 2)
  {
    Converter conv(args->url(0).url(), args->url(1).url());
    conv.convert();
  } else {
    kDebug() << "please give all parameters. Use --help to find out what is needed";
  }

  return 1;
}
