/***************************************************************************
 *   Copyright (C) 2009 by Wang Hoi <zealot.hoi@gmail.com>                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "koratmainwindowgraphics.h"

#include <plasma/framesvg.h>
#include <QGraphicsLinearLayout>

namespace Korat
{
class MainWindowGraphicsPrivate
{
public:
    MainWindowGraphicsPrivate(MainWindowGraphics *mainwindow)
        : q(mainwindow)
        , background(new Plasma::FrameSvg(mainwindow))
        , toolbar(0)
        , centralWidget(0)
        , statusbar(0)
        , layout(new QGraphicsLinearLayout(Qt::Vertical))
        , lrDockLayout(new QGraphicsLinearLayout())
        , centralLayout(new QGraphicsLinearLayout(Qt::Vertical))
    {
    }

    void updateLayout()
    {
        while (layout->count()) {
            layout->removeAt(0);
        }
        while (lrDockLayout->count()) {
            lrDockLayout->removeAt(0);
        }
        while (centralLayout->count()) {
            centralLayout->removeAt(0);
        }

        if (docks[Top]) {
            centralLayout->addItem(docks[Top]);
        }
        if (centralWidget) {
            centralWidget->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
            centralLayout->addItem(centralWidget);
        }
        if (docks[Bottom]) {
            centralLayout->addItem(docks[Bottom]);
        }

        if (docks[Left]) {
            lrDockLayout->addItem(docks[Left]);
        }
        lrDockLayout->addItem(centralLayout);
        if (docks[Right]) {
            lrDockLayout->addItem(docks[Right]);
        }

        if (toolbar) {
            layout->addItem(toolbar);
        }
        layout->addItem(lrDockLayout);
        if (statusbar) {
            layout->addItem(statusbar);
        }
    }

    MainWindowGraphics *q;

    Plasma::FrameSvg *background;

    ToolBar *toolbar;
    QGraphicsWidget *centralWidget;
    StatusBar *statusbar;

    QMap<Corner, QList<Drawer *> > drawers;
    QMap<Edge, Dock *> docks;

    QGraphicsLinearLayout *layout;
    QGraphicsLinearLayout *lrDockLayout;
    QGraphicsLinearLayout *centralLayout;

};

MainWindowGraphics::MainWindowGraphics(QGraphicsItem *parent)
    : QGraphicsWidget(parent)
    , d(new MainWindowGraphicsPrivate(this))
{
    d->background->setImagePath("dialogs/background");

    d->layout->setSpacing(0);
//    d->layout->setContentsMargins(0,0,0,0);

    d->lrDockLayout->setSpacing(0);
    d->lrDockLayout->setContentsMargins(0,0,0,0);

    d->centralLayout->setSpacing(0);

    d->lrDockLayout->addItem(d->centralLayout);
    d->layout->addItem(d->lrDockLayout);

    setLayout(d->layout);

    themeUpdated();
}

MainWindowGraphics::~MainWindowGraphics()
{

}

ToolBar *MainWindowGraphics::toolBar()
{
    return d->toolbar;
}

void MainWindowGraphics::setToolBar(ToolBar *bar)
{
    if (d->toolbar) {
        d->layout->removeAt(0);
        d->toolbar->hide();
        delete d->toolbar;
    }
    d->toolbar = bar;
//X     if (d->toolbar) {
//X         d->layout->addItem(d->toolbar);
//X     }
    d->updateLayout();
}

StatusBar *MainWindowGraphics::statusBar()
{
    return d->statusbar;
}

void MainWindowGraphics::setStatusBar(StatusBar *bar)
{
    d->statusbar = bar;
    d->updateLayout();
}

Drawer *MainWindowGraphics::drawer(Corner corner)
{
    if (d->drawers[corner].isEmpty()) {
        return 0;
    } else {
        return d->drawers[corner].last();
    }
}

void MainWindowGraphics::addDrawer(Drawer *widget, Corner corner)
{
    d->drawers[corner] << widget;
    d->updateLayout();
}

Dock *MainWindowGraphics::dock(Edge edge)
{
    return d->docks[edge];
}

void MainWindowGraphics::setDock(Dock *widget, Edge edge)
{
    d->docks[edge] = widget;
    d->updateLayout();
}

QGraphicsWidget *MainWindowGraphics::centralWidget()
{
    return d->centralWidget;
}

void MainWindowGraphics::setCentralWidget(QGraphicsWidget *widget)
{
    d->centralWidget = widget;
}

void MainWindowGraphics::resizeEvent(QGraphicsSceneResizeEvent * event)
{
    QGraphicsWidget::resizeEvent(event);
    d->background->resizeFrame(event->newSize());
}

void MainWindowGraphics::paint(QPainter *painter,const QStyleOptionGraphicsItem *option, QWidget *widget)
{
    Q_UNUSED(option);
    Q_UNUSED(widget);

    d->background->paintFrame(painter);
}

void MainWindowGraphics::themeUpdated()
{
    qreal left,top,right,bottom;
    d->background->getMargins(left,top,right,bottom);
    setContentsMargins(left,top,right,bottom);
}

} // namespace Korat

#include "koratmainwindowgraphics.moc"
// vim: sw=4 sts=4 et tw=100
