#include "widget.h"

#include "node.h"

#include <QMouseEvent>
#include <QVariant>

#include <qdebug.h>

using namespace Quasar;

Widget::Widget(const QString &imgFile)
    : QGLWidget(),
      m_image(imgFile),
      m_location(imgFile)
{
    m_image = QGLWidget::convertToGLFormat(m_image);
}

void Widget::initializeGL()
{
    glClearColor(0.3, 0.3, 0.3, 1);
    qDebug()<<"Image null = "<<m_image.isNull();
    qDebug()<<"\t w = "<<m_image.width()
            <<", h = "<<m_image.height();
#if 0
    m_tex = Texture((const char*)m_image.bits(),
                    m_image.width(), m_image.height(), GL_RGBA);

    qDebug()<<"Filters = "<<Composition::groups();
    m_node = m_compo.createNode("/color/saturation");
    Q_ASSERT(m_node);
    m_node->setPropertyValue("texture",
                             Texture((const char*)m_image.bits(),
                                     m_image.width(), m_image.height(),
                                     GL_RGBA));
    m_compo.prepare();
#else
    Node *input = m_compo.createNode("/input/file");
    Q_ASSERT(input);
    input->setPropertyValue("filename", m_location);

    Node *node = m_compo.createNode("/color/saturation");
    Q_ASSERT(node);
    Quasar::link(input, "texture", node, "texture");
    //node->setPropertyValue("alpha", 2);
    m_node = node;
    m_compo.prepare();
#endif
}

void Widget::paintGL()
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    m_compo.execute();
    glFlush();
    QVariant res = m_node->propertyValue("output");
    Texture tex = qvariant_cast<Texture>(res);

    qDebug()<<"Texture is "<<tex.id();
    glPushMatrix();
    glLoadIdentity();
    glTranslatef(width()/2-tex.width()/2,
                 height()/2-tex.height()/2,
                 0);
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, tex.id());
    glBegin(GL_TRIANGLE_STRIP);
    glTexCoord2f(0, 0); glVertex3f(0, 0, 0);
    glTexCoord2f(0, 1); glVertex3f(0, tex.height(), 0);
    glTexCoord2f(1, 0); glVertex3f(tex.width(), 0, 0);
    glTexCoord2f(1, 1); glVertex3f(tex.width(), tex.height(), 0);
    glEnd();
    glPopMatrix();

    glFlush();
}

void Widget::resizeGL(int width, int height)
{
    glViewport(0, 0, width, height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(0, width, 0, height, -999999, 999999);
    glMatrixMode(GL_MODELVIEW);
}

void Widget::mousePressEvent(QMouseEvent *event)
{
    QPointF pt = event->pos();

    m_point = QPointF(pt.x(), height() - pt.y());
    if (m_node) {
        //make it from 0 to 4
        m_node->setPropertyValue("alpha",
                                 pt.y()/(height()/4.));
    }
    update();
}

void Widget::mouseMoveEvent(QMouseEvent *event)
{
    QPointF pt = event->pos();

    m_point = QPointF(pt.x(), height() - pt.y());
    update();
}
