#include "propertyeditor.h"

#include "editors.h"

#include "node.h"

#include <QGroupBox>
#include <QVBoxLayout>
#include <QLayoutItem>
#include <QDebug>

PropertyEditor::PropertyEditor(QWidget *parent)
    : QWidget(parent),
      m_node(0)
{
    m_layout = new QVBoxLayout;
    m_layout->setAlignment(Qt::AlignTop);
    setLayout(m_layout);
}

void PropertyEditor::setNode(Quasar::Node *node)
{
    if (node == m_node)
        return;

    QLayoutItem *child;
    while ((child = m_layout->takeAt(0)) != 0) {
        delete child->widget();
        delete child;
    }

    m_node = node;
    QGroupBox *groupBox = new QGroupBox(node->name());
    groupBox->setSizePolicy(QSizePolicy::Minimum,
                            QSizePolicy::Minimum);
    QVBoxLayout *layout = new QVBoxLayout;
    QList<Quasar::Property> props = node->properties();

    foreach(Quasar::Property prop, props) {
        if (prop.kind == Quasar::Property::InputProperty) {
            WidgetEditor *editor = WidgetEditor::editorFor(prop);
            if (editor) {
                connect(editor, SIGNAL(changed(const QString&, const QVariant&)),
                        this, SLOT(changed(const QString&, const QVariant&)));
                QGroupBox *box = new QGroupBox(prop.name);
                box->setSizePolicy(QSizePolicy::Minimum,
                                   QSizePolicy::Fixed);
                QVBoxLayout *l = new QVBoxLayout;
                l->addWidget(editor);
                box->setLayout(l);
                layout->addWidget(box);
            }
        }
    }
    groupBox->setLayout(layout);
    m_layout->addWidget(groupBox);
}

Quasar::Node * PropertyEditor::node() const
{
    return m_node;
}

void PropertyEditor::changed(const QString &name,
                             const QVariant &val)
{
    qDebug()<<"+++ prop changed "<<name<<val;
    if (!m_node.isNull()) {
        m_node->setPropertyValue(name, val);
        emit edited();
    }
}
