#define GL_GLEXT_PROTOTYPES
#include "brightnessnode_p.h"

#include <QVariant>
#include <QDebug>

#include <GL/glext.h>
#include <GL/gl.h>

using namespace Quasar;


static const char *vertShaderText =
    "void main() {\n"
    "   gl_FrontColor = gl_Color;\n"
    "   gl_TexCoord[0] = gl_MultiTexCoord0;\n"
    "   gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;\n"
    "}\n";

static const char *fragShaderText =
    "uniform sampler2D srcTex;\n"
    "uniform float Alpha;"
    "void main() {\n"
    "    gl_FragColor = texture2D(srcTex,  gl_TexCoord[0].st) * Alpha;\n"
    "}\n";

BrightnessNode::BrightnessNode()
    : FilterNode("Brigtness")
{
    setShaderSources(vertShaderText, fragShaderText);
    Quasar::Property aprop = {
        Quasar::Property::InputProperty,

        QString::fromLatin1("Alpha"),
        QVariant::Double,
        1.0,
        QString::fromLatin1("Brightness alpha"),

        1.0,
        0,
        10,
    };
    addProperty(QString::fromLatin1("Alpha"), aprop);
}

void BrightnessNode::programChanged(GLuint prog)
{
    GLuint texLoc = glGetUniformLocationARB(programId(), "srcTex");
    glUniform1iARB(texLoc, 0);
    m_alphaLoc =  glGetUniformLocationARB(programId(), "Alpha");
    glUniform1fARB(m_alphaLoc, 1.0);
}


QRectF BrightnessNode::domainOfDefinition(const QRectF &rect) const
{
    return rect;
}

QRectF BrightnessNode::regionOfInterest(const QRectF &rect) const
{
    return rect;
}

void BrightnessNode::aboutToExecute()
{
}
