/*
    This file is part of the KDE project.
    Copyright (c) 2005 Friedrich W. H. Kossebau <Friedrich.W.H@Kossebau.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License version 2 as published by the Free Software Foundation.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301  USA
*/

// qt specific
#include <qpopupmenu.h>
// kde specific
// #include <dcopclient.h>
#include <kapplication.h>
#include <kglobal.h>
#include <klocale.h>
#include <kiconloader.h>
#include <krun.h>
#include <kdebug.h>
#include <kaboutdata.h>
#include <kaboutapplication.h>
#include <kmessagebox.h>
// applet specific
#include "menubutton.h"
#include "ksessionmenubuttonfiller.h"
#include "sessionapplet.h"


static const char GroupGeneralId[] =  "General";

const char * const SessionApplet::AppletName = "sessionapplet";

SessionApplet::SessionApplet( const QString& ConfigFile, QWidget *Parent, const char *Name)
: KPanelApplet( ConfigFile, KPanelApplet::Normal, About|Preferences, Parent, Name ),
  SessionManager( kapp->desktop()->screen(kapp->desktop()->screenNumber( this )), qt_xscreen() )
{
    if( !Parent )
        setBackgroundMode( X11ParentRelative );

    Button = new MenuButton( new KSessionMenuButtonFiller(KUser(),&SessionManager), this );
    Button->setPanelPosition( position() );
}

void SessionApplet::about()
{
    KAboutData Data( AppletName,
                     I18N_NOOP("Session Applet"),
                     "0.4.0",
                     I18N_NOOP("Lets the user switch the sessions."),
                     KAboutData::License_LGPL_V2,
                     "(c) 2005, Friedrich W. H. Kossebau" );

    Data.addAuthor( "Friedrich W. H. Kossebau",
                    I18N_NOOP("Author"),
                    "Friedrich.W.H@Kossebau.de" );

    KAboutApplication Dialog( &Data );
    Dialog.exec();
}


void SessionApplet::loadConfig()
{
    KConfig *Config = config();
    Config->setGroup( GroupGeneralId );

//     ExcludeMyself = Config->readBoolEntry( ExcludeMyselfId, false );
}


void SessionApplet::saveConfig()
{
    KConfig *Config = config();
    Config->setGroup( GroupGeneralId );

//     Config->writeEntry( ExcludeMyselfId, ExcludeMyself );

    Config->sync();
}

void SessionApplet::preferences()
{
    KMessageBox::information(0, i18n("This is a preferences box"));
/*    PreferencesDialog dialog(mMedia);

    dialog.setExcludedMediumTypes(mExcludedTypesList);
    dialog.setExcludedMedia(mExcludedList);

    if(dialog.exec())
    {
        mExcludedTypesList = dialog.excludedMediumTypes();
        mExcludedList = dialog.excludedMedia();
        saveConfig();
        reloadList();
    }*/
}


void SessionApplet::resizeEvent( QResizeEvent * )
{
    int Size = (orientation()==Vertical)? width() : height();

    Button->resize( Size, Size );
}


int SessionApplet::widthForHeight( int Height ) const
{
    return Height;
}

int SessionApplet::heightForWidth( int Width ) const
{
    return Width;
}

void SessionApplet::positionChange( Position P )
{
    Button->setPanelPosition( P );
}

void SessionApplet::mousePressEvent( QMouseEvent *ME )
{
    if (!kapp->authorizeKAction("kicker_rmb"))
        return;

    if( ME->button() == RightButton )
    {
        QPopupMenu *popup = new QPopupMenu();
        popup->insertItem( SmallIcon( "configure" ),
                            i18n( "&Configure Session Manager..." ),
                            this, SLOT( slotLogoutPrefs() ) );
        popup->insertItem( SmallIcon( "configure" ),
                            i18n( "&Configure Screen Saver..." ),
                            this, SLOT( slotLockPrefs() ) );
        popup->exec( ME->globalPos() );
        delete popup;
    }
}

void SessionApplet::slotLockPrefs()
{
    // Run the screensaver settings
    KRun::run( "kcmshell screensaver", KURL::List() );
}

void SessionApplet::slotLogoutPrefs()
{
    // Run the logout settings.
    KRun::run( "kcmshell kcmsmserver", KURL::List() );
}


SessionApplet::~SessionApplet()
{
    KGlobal::locale()->removeCatalogue( AppletName );
}

#include "sessionapplet.moc"
